=begin
(c) 2011 TIG & Refractive Software
All Rights Reserved.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING,WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Octane  - Exporter / Renderer Tool

Usage: menu File > 'Octane Render...'
       or Context-menu 'Octane Render...' [if item enabled in code]
       or Toolbar 'Octane Render'
       or type 'octane' in the Ruby Console.

It opens the 'Octane Render' dialog, which exports SKP to OBJ/MTL/Textures
format [for Octane], with other camera, sun info etc if desired, it can 
then 'render' an image using Octane, or make an 'animation' of frame images...

Version:
[full]
1024.
01 20110314 Suits v1024, but should work on earlier versions. First issue.
02 20110323 Glitch with 'smoothed' surface export addressed.
03 20110513 Support for PC mapped-drive locations added.
04 20110514 Motion-blur errors trapped when no leading or trailing frames.
TIG

=end
###
require 'sketchup.rb'
require 'deBabelizer.rb'
###

class Sketchup::Image
	if not Sketchup::Image.method_defined?(:definition)
      def definition()
        if not self.valid?
            puts "Image "+self.to_s+" is no longer valid."
            return false
        end#if
		self.model.definitions.each { |d| return d if d.image? && d.instances.include?(self)}
        return nil
	  end#def
    end#if

    if not Sketchup::Image.method_defined?(:transformation)
	  def transformation()
        if not self.valid?
            puts "Image "+self.to_s+" is no longer valid."
            return false
        end#if
		origin = self.origin
		axes = self.normal.axes
		tr = Geom::Transformation.axes(ORIGIN, axes.at(0), axes.at(1), axes.at(2))
		tr = tr*Geom::Transformation.rotation(ORIGIN, Z_AXIS, self.zrotation)
		tr = (tr*Geom::Transformation.scaling(ORIGIN, self.width/self.pixelwidth, self.height/self.pixelheight, 1)).to_a
		tr[12] = origin.x
		tr[13] = origin.y
		tr[14] = origin.z
		return Geom::Transformation.new(tr)
	  end#def
    end#if

	if not Sketchup::Image.method_defined?(:transformation=)
      def transformation=(tr)
        if not self.valid?
            puts "Image "+self.to_s+" is no longer valid."
            return false
        end#if
        if tr.class==Array
          tr=Geom::Transformation.new(tr)
        end#if
		status=self.transform!(self.transformation.inverse * tr)
        if status
          return self
        else
          return nil
        end#if
	  end
    end#if    
end# Image class

class Sketchup::Group
    # Sometimes the group.entities.parent refer to the wrong definition.
    # This checks for error and locates the correct parent definition.
    if not Sketchup::Group.method_defined?(:definition)
        def definition()
          if self.entities.parent.instances.include?(self)
            return self.entities.parent
          else
            Sketchup.active_model.definitions.each { |definition|
                return definition if definition.instances.include?(self)
            }
          end
          return nil # Should not happen.
        end
    end#if
end# group class

###########

class Octane

### Dir.chdir(path) hack
def dirChdir(path)
  return false if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return Dir.chdir(path)
end#if

### Dir.pwd hack
def dirPwd()
  path=Dir.pwd
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return path
end#def

### Dir.mkdir hack
def dirMkdir(path)
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return Dir.mkdir(path)
end#def

### File.exist?(path) hack
def fileExist?(path=nil)
  return false if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.exist?(path)
end#def

### File.dirname(path) hack
def fileDirname(path=nil)
  return "." if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.dirname(path)
end#def

### File.join(path,file)path hack
def fileJoin(path=nil,file=nil)
  return "" if not path
  return path if not file
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
    file=file.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.join(path,file)
end#def

### File.delete(path) hack
def fileDelete(path=nil)
  return false if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.delete(path)
end#def

### File.basename(path, ext) hack
def fileBasename(path=nil, extn=nil)
  return "" if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
    extn=extn.unpack("U*").map{|c|c.chr}.join if extn
  end#if
  return File.basename(path, extn) if extn
  return File.basename(path)if not extn
end#def

### File.new(path, optn) hack
def fileNew(path=nil,optn=nil)
  return false if not path
  optn="" if not optn
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
    optn=optn.unpack("U*").map{|c|c.chr}.join if optn
  end#if
  return File.new(path,optn)if optn
  return File.new(path)if not optn
end#def

### File.chmod(0777, path) hack
def fileChmod(nnnn=nil,path=nil)
  return false if not nnnn
  return false if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.chmod(nnnn,path)
end#def

### File.extname(opu) hack
def fileExtname(path=nil)
  return "" if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.extname(path)
end#def

### File.directory?(file) hack
def fileDirectory?(path=nil)
  return false if not path
  if @pc and $KCODE=="UTF8"
    path=path.unpack("U*").map{|c|c.chr}.join
  end#if
  return File.directory?(path)
end#def

###############################################

### deBabelizer code ##########################
def db(string)
  dir=fileJoin(fileDirname(__FILE__), "OctaneLingvos")
  locale=Sketchup.get_locale.upcase
  path=fileJoin(dir, ("Octane"+locale+".lingvo"))
  if not fileExist?(path)
    return string
  else
    deBabelizer(string, path)
  end
end#def

def Octane::db(string)
  dir=File.join(File.dirname(__FILE__), "OctaneLingvos")
  locale=Sketchup.get_locale.upcase
  path=File.join(dir, ("Octane"+locale+".lingvo"))
  if not File.exist?(path)
    return string
  else
    deBabelizer(string, path)
  end
end#if
############################################

    @@idset = %w(octane_render_binary project_path project_name native_unit_size resolution width height percentage export_camera active_camera lens_aperture_radius lens_aperture_radius_value focal_depth focal_depth_value motion_blur motion_blur_value export_sun_direction light_source gpu_devices gpu_device0 gpu_device1 gpu_device2 gpu_device3 image_output start_frame end_frame step pixel_samples pixs)

	@@dlg=UI::WebDialog.new(" Refractive Software 2011",false,"Octane",600,740,25,25,false) ###true=resizeable

	@@native_unit_size=(db("meters"))
    
	@@resolution="false"
    @@percentage="100%"
    
    @@export_camera="true"
    @@active_camera=(db("from <Current View>"))
    @@lens_aperture_radius="false"
    @@lens_aperture_radius_value=1.0
    @@focal_depth="false"
    @@focal_depth_value=5.0
    @@motion_blur="false"
    @@motion_blur_value=(db("Next"))
    
    @@export_sun_direction="false"
    @@light_source=(db("Sun from <Current View>"))
    
    @@gpu_devices="false"
    @@gpu_device0="false"
    @@gpu_device1="false"
    @@gpu_device2="false"
    @@gpu_device3="false"

    @@image_output=""
    
    @@pixel_samples=(db("Medium (256 s/px)"))
    @@pixel_samples_array=[(db("Very Low (8 s/px)")), (db("Low (32 s/px)")), (db("Medium (256 s/px)")), (db("Medium/High (512 s/px)")), (db("High (1024 s/px)")), (db("Very High (4096 s/px)")), (db("Custom"))]
    @@pixs="256"

### end protected

  def initialize

    @msg=""
    @model=Sketchup.active_model
	@sel=@model.selection
    
    @dlg = @@dlg
    
	@octane_render_binary=Sketchup.read_default("Octane", "octane_render_binary", "").tr("\\\\", "\\")
    
	@project_path=@model.get_attribute("Octane", "project_path", nil)
	@project_name=@model.get_attribute("Octane", "project_name", nil)
    
	@native_unit_size=@@native_unit_size
        
	@resolution=@model.get_attribute("Octane", "resolution", @@resolution)
    
    @width=@model.get_attribute("Octane", "width", @model.active_view.vpwidth.to_s)
    @height=@model.get_attribute("Octane", "height", @model.active_view.vpheight.to_s)
    @percentage=@model.get_attribute("Octane", "percentage", @@percentage)
    @percentages_array=["100%","75%","50%","25%",(db("Reset"))]
       
    @export_camera=@model.get_attribute("Octane", "export_camera", @@export_camera)
    @active_camera=@model.get_attribute("Octane", "active_camera", @@active_camera)
    @lens_aperture_radius=@model.get_attribute("Octane", "lens_aperture_radius", @@lens_aperture_radius)
    @lens_aperture_radius_value=@model.get_attribute("Octane", "lens_aperture_radius_value", @@lens_aperture_radius_value)
    @focal_depth=@model.get_attribute("Octane", "focal_depth", @@focal_depth)
    @focal_depth_value=@model.get_attribute("Octane", "focal_depth_value", @@focal_depth_value)
    @motion_blur=@model.get_attribute("Octane", "motion_blur", @@motion_blur)
    @motion_blur_value=@model.get_attribute("Octane", "motion_blur_value", @@motion_blur_value)
      
    @export_sun_direction=@model.get_attribute("Octane", "export_sun_direction", @@export_sun_direction)
    @light_source=@model.get_attribute("Octane", "light_source", @@light_source)
        
    @gpu_devices=@model.get_attribute("Octane", "gpu_devices", @@gpu_devices)
    @gpu_device0=@model.get_attribute("Octane", "gpu_device0", @@gpu_device0)
    @gpu_device1=@model.get_attribute("Octane", "gpu_device1", @@gpu_device1)
    @gpu_device2=@model.get_attribute("Octane", "gpu_device2", @@gpu_device2)
    @gpu_device3=@model.get_attribute("Octane", "gpu_device3", @@gpu_device3)
       
    @image_output=@model.get_attribute("Octane", "image_output", nil)
    
    self.get_scenes()
    
    @start_frame=nil
    @start_frame=@scenes[0]if @scenes[0]
    if @scenes.include?(@model.get_attribute("Octane", "start_frame", "()"))
      @start_frame=@model.get_attribute("Octane", "start_frame", nil)
    end#if
    @end_frame=nil
    @end_frame=@scenes[1]if @scenes[1]
    if @scenes.include?(@model.get_attribute("Octane", "end_frame", "()"))
      @end_frame=@model.get_attribute("Octane", "end_frame", nil)
    end#if
    @step=@model.get_attribute("Octane", "step", nil)
    
    @pixel_samples=@model.get_attribute("Octane", "pixel_samples", @@pixel_samples)
    @pixel_samples_array=@@pixel_samples_array
    @pixs=@model.get_attribute("Octane", "pixs", @@pixs)
    
    @anims2go=[]

	@dlg.add_action_callback("browse_octane_render_binary"){|d,p|self.browse_octane_render_binary()}
 	@dlg.add_action_callback("browse_project_path"){|d,p|self.browse_project_path()}
    @dlg.add_action_callback("use_existing_project"){|d,p|self.use_existing_project()}
    @dlg.add_action_callback("update_project_name"){|d,p|self.update_project_name()}

    @dlg.add_action_callback("resolution"){|d,p|self.resolution(p)}
    @dlg.add_action_callback("percentage_changed"){|d,p|self.percentage_changed()}
        
    @dlg.add_action_callback("export_camera"){|d,p|self.export_camera(p)}
    @dlg.add_action_callback("set_camera"){|d,p|self.set_camera()}
    @dlg.add_action_callback("lens_aperture_radius"){|d,p|self.lens_aperture_radius(p)}
    @dlg.add_action_callback("focal_depth"){|d,p|self.focal_depth(p)}
    @dlg.add_action_callback("motion_blur"){|d,p|self.motion_blur(p)}
      
    @dlg.add_action_callback("export_sun_direction"){|d,p|self.export_sun_direction(p)}
        
    @dlg.add_action_callback("gpu_devices"){|d,p|self.gpu_devices(p)}
    @dlg.add_action_callback("gpu_device0"){|d,p|self.gpu_device0(p)}
    @dlg.add_action_callback("gpu_device1"){|d,p|self.gpu_device1(p)}
    @dlg.add_action_callback("gpu_device2"){|d,p|self.gpu_device2(p)}
    @dlg.add_action_callback("gpu_device3"){|d,p|self.gpu_device3(p)}
        
	@dlg.add_action_callback("export"){|d,p|self.export()}
	@dlg.add_action_callback("render"){|d,p|self.render()}
        
    @dlg.add_action_callback("select_image_output"){|d,p|self.select_image_output()}
    @dlg.add_action_callback("render_animation"){|d,p|self.render_animation()}
    
    @dlg.add_action_callback("update_start"){|d,p|self.update_start()}
    @dlg.add_action_callback("update_end"){|d,p|self.update_end()}
    @dlg.add_action_callback("update_step"){|d,p|self.update_step()}
    
    @dlg.add_action_callback("update_pixs"){|d,p|self.update_pixs()}
    @dlg.add_action_callback("update_pixs_txt"){|d,p|self.update_pixs_txt()}
    
    @dlg.add_action_callback("help"){|d,p|self.help()}
    
    @dlg.add_action_callback("set_button"){|d,p|self.set_button()}
        
	###@dlg.add_action_callback("populate"){|d,p|self.populate()}
    @dlg.set_on_close{self.deactivate(false)}
    ### 
  end

  def activate
    @pc=false
    @pc=true if [PLATFORM].grep(/mswin/)==[PLATFORM]
    ### set up project
    @path=@model.path ### puts"'"+Sketchup.active_model.path+"'"
    if not @path or @path==""
       UI.messagebox((db("Octane Render:"))+"\n"+(db("You must save this new Sketchup model before using this tool!")))
       self.deactivate(false)
       return nil
    end#if
    ### check on exe
    if not @octane_render_binary or @octane_render_binary==""
      UI.messagebox((db("Octane Render:"))+"\n"+(db("The 'Octane Render Binary' must be set...")))
      self.browse_octane_render_binary()
    elsif not fileExist?(@octane_render_binary.tr("\\", "/"))
      UI.messagebox((db("Octane Render:"))+"\n"+(db("The 'Octane Render Binary' is not valid...")))
      self.browse_octane_render_binary()
    end#if
    @octane_render_binary.tr("/", "\\")if @pc
    ###
    project_path=fileDirname(@path)
    @project_path=project_path if not @project_path or @project_path==""
    title=@model.title
    project_name=title.tr(" ", "_").gsub(/[^A-Za-z0-9_\-]/, '_')
    @project_name=project_name if not @project_name or @project_name==""
    @project_name.tr!(" ", "_")
    @project_name="X" if @project_name==""
    image_output=@project_path if not @image_output or @image_output==""
    @image_name="img00001.png"
    @image_output=fileJoin(image_output, @image_name) if not @image_output or @image_output==""###
    ###
    @project_path.tr!("/", "\\")if @pc
    @image_output.tr!("/", "\\")if @pc
    ###
    self.get_scenes()
    ### now do dialog
    self.dialog()
  end#activate
  
  def deactivate(open=true)
    begin
      self.getIDvalues()if @dlg and open
      self.saveIDvalues()if @dlg and open
      @dlg.close() if open
      @dlgfb.close() if @dlgfb
      @anims2go.each{|f|fileDelete(f)if fileExist?(f)}if @anims2go[0]
      fileDelete(@cmd)if @cmd and fileExist?(@cmd)
      fileDelete(@acmd)if @acmd and fileExist?(@acmd)
    rescue
      puts 'Error on closing Octane Render Dialog!'
    end
    Sketchup.send_action("selectSelectionTool:")
    return nil
  end#deactivate
  
  def dialog()
    dialog_path=fileJoin(fileDirname(__FILE__), "OctaneDialog.html")
    if not fileExist?(dialog_path)
      UI.messagebox(dialog_path+"\n\n"+"MISSING HTML ???")
      return nil
    end#if
    ### remove unused versions of earlier HTMLs ###<<<<<<<<<<<<<<<<<<<<<<<<
    fileDelete(fileJoin(fileDirname(__FILE__), "/OctaneDialogWIN.html"))if fileExist?(fileJoin(fileDirname(__FILE__), "OctaneDialogWIN.html"))
    fileDelete(fileJoin(fileDirname(__FILE__), "OctaneDialogMAC.html"))if fileExist?(fileJoin(fileDirname(__FILE__), "OctaneDialogMAC.html"))
    ### ##########################################<<<<<<<<<<<<<<<<<<<<<<<<<
	#@dlg=@@dlg unless @dlg
    @dlg.set_file(dialog_path)
    ### make dialog for PC or MAC
    ### v09 BOTH Modal to suit SUp-v8
    if @pc
      @dlg.show_modal{
        self.populate() ###<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      }
    else
      @dlg.show_modal{
        self.populate() ###<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
      }
    end#if
    ###
  end#dialog
  
  def populate()
    ### Translate HTML values
    js=("document.getElementById(\"project_configuration_label\").setAttribute(\"innerHTML\",\""+(db("Project Configuration"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"octane_render_binary_label\").setAttribute(\"innerHTML\",\""+(db("Octane Render Binary"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"browse_octane_render_binary\").setAttribute(\"value\",\""+(db("Browse"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"project_path_label\").setAttribute(\"innerHTML\",\""+(db("Project Path"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"browse_project_path\").setAttribute(\"value\",\""+(db("Browse"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"project_name_label\").setAttribute(\"innerHTML\",\""+(db("Project Name"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"use_existing_project\").setAttribute(\"value\",\""+(db("Use Existing Project"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"native_unit_size_label\").setAttribute(\"innerHTML\",\""+(db("Native Unit Size"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"help\").setAttribute(\"value\",\""+(db("Help"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"export_configuration_label\").setAttribute(\"innerHTML\",\""+(db("Export Configuration"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"film_label\").setAttribute(\"innerHTML\",\"&nbsp;"+(db("Film"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"resolution_label\").setAttribute(\"innerHTML\",\""+(db("Resolution"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"width_label\").setAttribute(\"innerHTML\",\""+(db("Width"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"height_label\").setAttribute(\"innerHTML\",\""+(db("Height"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"camera_label\").setAttribute(\"innerHTML\",\"&nbsp;"+(db("Camera"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"export_camera_label\").setAttribute(\"innerHTML\",\""+(db("Export Camera"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"active_camera_label\").setAttribute(\"innerHTML\",\""+(db("Active Camera"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"lens_aperture_radius_label\").setAttribute(\"innerHTML\",\""+(db("Lens Aperture Radius"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"focal_depth_label\").setAttribute(\"innerHTML\",\""+(db("Focal Depth"))+" (m)\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"motion_blur_label\").setAttribute(\"innerHTML\",\""+(db("Motion Blur&nbsp;&nbsp;&nbsp;&nbsp;Interpolate"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"daylight_environment_label\").setAttribute(\"innerHTML\",\"&nbsp;"+(db("Daylight Environment"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"export_sun_direction_label\").setAttribute(\"innerHTML\",\""+(db("Export Sun Direction"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"light_source_label\").setAttribute(\"innerHTML\",\""+(db("Light Source"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"gpu_devices_label\").setAttribute(\"innerHTML\",\"&nbsp;"+(db("GPU Devices"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"specify_gpu_devices_label\").setAttribute(\"innerHTML\",\""+(db("Specify GPU Devices"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"export_label\").setAttribute(\"innerHTML\",\""+(db("Export"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"frame_label\").setAttribute(\"innerHTML\",\"&nbsp;&nbsp;"+(db("Frame"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"export_button_label\").setAttribute(\"value\",\""+(db("Export OBJ/MTL Only"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"render_button_label\").setAttribute(\"value\",\""+(db("Render Frame"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"animation_label\").setAttribute(\"innerHTML\",\"&nbsp;&nbsp;"+(db("Animation"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"image_output_label\").setAttribute(\"innerHTML\",\""+(db("Image Output"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"start_frame_label\").setAttribute(\"innerHTML\",\""+(db("Start Frame"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"end_frame_label\").setAttribute(\"innerHTML\",\""+(db("End Frame"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"step_label\").setAttribute(\"innerHTML\",\""+(db("Step"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"pixel_samples_label\").setAttribute(\"innerHTML\",\""+(db("Pixel Samples"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"select_image_output_button\").setAttribute(\"value\",\""+(db("Browse"))+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"render_animation_button\").setAttribute(\"value\",\""+(db("Render Animation"))+"\");")
    @dlg.execute_script(js)
    ###
    #set octane_render_binary path
    octane_render_binary=@octane_render_binary.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"octane_render_binary\").setAttribute(\"value\",\""+octane_render_binary+"\");")
    @dlg.execute_script(js)
    #set project_path
    project_path=@project_path.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"project_path\").setAttribute(\"value\",\""+project_path+"\");")
    @dlg.execute_script(js)
    #set project_name
    project_name=@project_name
    js=("document.getElementById(\"project_name\").setAttribute(\"value\",\""+project_name+"\");")
    @dlg.execute_script(js)
    #set extg native_unit_size
    units=@model.options["UnitsOptions"]["LengthUnit"]
    case units
      when 1
        @native_unit_size=(db("feet"))
      when 2
        @native_unit_size=(db("millimeters"))
      when 3
        @native_unit_size=(db("centimeters"))
      when 4
        @native_unit_size=(db("meters"))
      else ###0
        @native_unit_size=(db("inches"))
    end#case
    #set native_units
    units_array=[(db("inches")),(db("feet")),(db("meters")),(db("centimeters")),(db("millimeters"))]
	units_array.each_with_index do |unit,i|
		js="document.getElementById(\"native_unit_size\")[#{i}] = new Option('#{unit}','#{unit}'"
		js+=",true,true" if @native_unit_size==unit ### SELECTED UNITS
		js+=");"
		@dlg.execute_script(js)
	end#do
    #set resolution
    if @resolution=="true"
      js=("document.getElementById(\"resolution\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    # h/w
    if @width.to_i==0 or @height.to_i==0
      @width=@model.active_view.vpwidth.to_s
      @height=@model.active_view.vpheight.to_s
      @percentage="100%"
    end#if
    #set h/w
    js=("document.getElementById(\"width\").setAttribute(\"value\",\""+@width+"\");")
    @dlg.execute_script(js)
    js=("document.getElementById(\"height\").setAttribute(\"value\",\""+@height+"\");")
    @dlg.execute_script(js)
    #set percentage
	@percentages_array.each_with_index do |p,i|
		js="document.getElementById(\"percentage\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @percentage==p ### SELECTED Percent
		js+=");"
		@dlg.execute_script(js)
	end#do
    ### always have export_camera set and disabled
    #set export_camera could be omitted as we must always export a camera to render ? NO
    if @export_camera=="true"
      js=("document.getElementById(\"export_camera\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"active_camera\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    #set active_camera
    @cameras_array=[(db("from <Current View>"))]
    ### check for scenes
    @pages=[]
    @model.pages.each{|page|
      name=page.name
      #name=page.label if name != page.label ### NO in anim only ?
      @pages<< name
    }
    @cameras_array<< (db("from <Current Scene>")) if @pages[0]
    @cameras_array=@cameras_array+@pages
    if not @cameras_array.include?(@active_camera)
      @active_camera=@cameras_array[0]
    end#if
    ###
	@cameras_array.each_with_index do |p,i|
		js="document.getElementById(\"active_camera\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @active_camera==p ### SELECTED camera
		js+=");"
		@dlg.execute_script(js)
	end#do
    ### zoom to this view
    if @active_camera==@cameras_array[0]
      ###do noting
    elsif @active_camera==@cameras_array[1]
      @model.active_view.camera=@model.pages.selected_page.camera
    else
      @model.pages.selected_page=@model.pages[@active_camera]
    end#if
    #set lens_aperture_radius
    if @lens_aperture_radius=="true"
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    #set lens_aperture_radius
    js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"value\",\""+@lens_aperture_radius_value.to_s+"\");")
    @dlg.execute_script(js)
    #set focal_depth
    if @focal_depth=="true"
      js=("document.getElementById(\"focal_depth\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    #set focal_depth_value
    js=("document.getElementById(\"focal_depth_value\").setAttribute(\"value\",\""+@focal_depth_value.to_s+"\");")
    @dlg.execute_script(js)
    #set motion_blur
    if @motion_blur=="true"
      js=("document.getElementById(\"motion_blur\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    #set motion_blur_value
    motion_blur_array=[(db("Next")),(db("Previous"))]
	motion_blur_array.each_with_index do |p,i|
		js="document.getElementById(\"motion_blur_value\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @motion_blur_value==p ### SELECTED motion_blur_value
		js+=");"
		@dlg.execute_script(js)
	end#do
    #set export_sun_direction
    if @export_sun_direction=="true"
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    #set light_sources
    @light_source=@dlg.get_element_value("light_source")
    @lights_array=[(db("Sun from <Current View>"))]
    ### check for scenes
    @pages=[]
    @model.pages.each{|page|
      name=page.name
      ###name=page.label if name != page.label
      @pages<< name
    }
    @lights_array<< (db("Sun from <Current Scene>")) if @pages[0]
    @lights_array=@lights_array+@pages
    if not @lights_array.include?(@light_source)
      @light_source=@lights_array[0]
    end#if
    ###
	@lights_array.each_with_index do |p,i|
		js="document.getElementById(\"light_source\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @light_source==p ### SELECTED light
		js+=");"
		@dlg.execute_script(js)
	end#do
    @light_source=@dlg.get_element_value("active_camera")
    #set gpu_devices <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    if @gpu_device0=="true"
      if @gpu_devices=="true"
        js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",\"checked\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"true\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device0\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
      else
        js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device0\").setAttribute(\"disabled\",\"disabled\");")
        @dlg.execute_script(js)
        @gpu_device0=="false"
      end#if
    else
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
    if @gpu_device1=="true"
      if @gpu_devices=="true"
        js=("document.getElementById(\"gpu_device1\").setAttribute(\"checked\",\"checked\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"true\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device1\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
      else
        js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",\"disabled\");")
        @dlg.execute_script(js)
        @gpu_device1=="false"
      end#if
    else
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
    if @gpu_device2=="true"
      if @gpu_devices=="true"
        js=("document.getElementById(\"gpu_device2\").setAttribute(\"checked\",\"checked\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"true\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device2\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
      else
        js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",\"disabled\");")
        @dlg.execute_script(js)
        @gpu_device2=="false"
      end#if
    else
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
    if @gpu_device3=="true"
      if @gpu_devices=="true"
        js=("document.getElementById(\"gpu_device3\").setAttribute(\"checked\",\"checked\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"true\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device3\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
      else
        js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",\"disabled\");")
        @dlg.execute_script(js)
        @gpu_device3=="false"
      end#if
    else
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
    ###
    if @gpu_devices=="true"
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      @gpu_device0=="false"
      @gpu_device1=="false"
      @gpu_device2=="false"
      @gpu_device3=="false"
    end#if
    #set image_output
    image_output=@image_output.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"image_output\").setAttribute(\"value\",\""+image_output+"\");")
    @dlg.execute_script(js)
    #set start_frame etc
    self.update_start(true)
    if @start_frame and @start_frame!=""
      js=("document.getElementById(\"start_frame\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    ###
    self.update_end(true)
    if @end_frame and @end_frame!=""
      js=("document.getElementById(\"start_frame\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    ###
    self.update_step(true)
    if @step and @step!=""
      @step=@dlg.get_element_value("step")
      js=("document.getElementById(\"start_frame\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"start_frame\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"end_frame\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"step\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    ###
    #set pixrelsamples
    pixel_samples_array=@pixel_samples_array
	pixel_samples_array.each_with_index do |p,i|
		js="document.getElementById(\"pixel_samples\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @pixel_samples==p ### SELECTED pixel_samples
		js+=");"
		@dlg.execute_script(js)
	end#do
    ### pixs
    @pixel_samples=@dlg.get_element_value("pixel_samples")
    pixsa=@pixel_samples
    if pixsa==(db("Custom"))
      pixs=@pixs
    else
      pixs=pixsa.split("(")[1].to_i.to_s
    end#if    
    js=("document.getElementById(\"pixs\").setAttribute(\"value\",\""+pixs+"\");")
    @dlg.execute_script(js)
    @pixs==@dlg.get_element_value("pixs")
    ### trap for start_frame etc
    if not @start_frame or not @end_frame or not @step
      js=("document.getElementById(\"pixel_samples\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"pixs\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
    ###
  end#populate  
  
  def browse_octane_render_binary()
    Sketchup.set_status_text((db("Octane Render:"))+(db("Browse for 'Octane Render Binary'...")))
    @octane_render_binary=@dlg.get_element_value("octane_render_binary")if @dlg.visible?
    ### browse for file
    binary=UI.openpanel((db("Choose the 'Refractive Software' - 'Octane Render Binary'...")), nil, "*.exe")
    special=false
    special=true if @pc and @octane_render_binary.length != @octane_render_binary.unpack("C*").pack("U*").length
    if special
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Octane Render Binary'!"))+"\n"+(db("NO accented or special-characters allowed.")))
      self.deactivate()
      return nil
    end#if
    if not binary and not fileExist?(@octane_render_binary.tr("\\", "/"))
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Octane Render Binary'!")))
      self.deactivate()
      return nil
    end#if
    if binary
      binary.tr!("/", "\\")if @pc ### fix for PC openpanel - file seperator on PC = \\, but Mac = /
      @octane_render_binary=binary
      octane_render_binary=binary.gsub(/\\/){'\\\\'}
      Sketchup.write_default("Octane", "octane_render_binary", octane_render_binary)
    end#if
    ###
    octane_render_binary=@octane_render_binary.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"octane_render_binary\").setAttribute(\"value\",\""+octane_render_binary+"\");")
    @dlg.execute_script(js)
    Sketchup.set_status_text(db("Octane Render:"))
  end#browse_octane_render_binary
  
  def browse_project_path()
    Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Browse 'Project Path' Folder...")))
    ###
    @project_path=@dlg.get_element_value("project_path")if @dlg.visible?
    ### browse for folder
    if fileExist?(@project_path)
      self.startfb(@project_path, "project")
    else
      p pwd=dirPwd
      self.startfb(pwd, "project")
    end#if
    Sketchup.set_status_text(db("Octane Render:"))
    ###
  end#browse_project_path
  
  def update_project()
    project = @octane_render_folder_browser_result
    #UI.openpanel("Choose any file in a folder to select the 'Project Folder'...", @project_path)
    special=false
    special=true if @pc and project.length != project.unpack("C*").pack("U*").length
    if special
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Project Path'!"))+"\n"+(db("NO accented or special-characters allowed.")))
      self.deactivate()
      return nil
    end#if
    if not project and not fileExist?(@project_path.tr("\\", "/"))
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Project Path'!")))
      self.deactivate()
      return nil
    end#if
    @project_path=project
    @project_path.tr!("/", "\\")if @pc
    @model.set_attribute("Octane", "project_path", @project_path)
    project_path=@project_path.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"project_path\").setAttribute(\"value\",\""+project_path+"\");")
    @dlg.execute_script(js)
    Sketchup.set_status_text(db("Octane Render:"))
  end#update_project
  
  def use_existing_project()
    @project_path=@dlg.get_element_value("project_path")if @dlg.visible?
    Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Use Existing Project...")))
    ### browse for file
    project=UI.openpanel((db("Choose 'Existing Project'...")), nil, "*.ocs")
    return nil if not project
    special=false
    special=true if @pc and project.length != project.unpack("C*").pack("U*").length
    if special
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Project Path'!"))+"\n"+(db("NO accented or special-characters allowed.")))
      self.deactivate()
      return nil
    end#if
    @project_path=fileDirname(project.tr("\\", "/"))
    @project_path.tr!("/", "\\")if @pc
    @model.set_attribute("Octane", "project_path", @project_path)
    project_path=@project_path.gsub(/\\/){'\\\\'}
    js=("document.getElementById(\"project_path\").setAttribute(\"value\",\""+project_path+"\");")
    @dlg.execute_script(js)
    @project_name=fileBasename(project.tr("\\", "/"), ".ocs")
    @project_name.tr!(" ", "_")
    @model.set_attribute("Octane", "project_name", @project_name)
    project_name=@project_name
    js=("document.getElementById(\"project_name\").setAttribute(\"value\",\""+project_name+"\");")
    @dlg.execute_script(js)
    Sketchup.set_status_text(db("Octane Render:"))
  end#use_existing_project
  
  def update_project_name()
    @project_name=@dlg.get_element_value("project_name")
    @model.set_attribute("Octane", "project_name", @project_name)
  end#update_project_name
  
  def resolution(p)
    @resolution=p
    if @resolution=="true"
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"resolution\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"width\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"height\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"percentage\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
  end#resolution
  
  def percentage_changed()
    @percentage=@dlg.get_element_value("percentage")
    ### check if w/h==0 and reset if do...
    if @percentage==(db("Reset"))
      @width=@model.active_view.vpwidth.to_s
      @height=@model.active_view.vpheight.to_s
      @percentage="100%"
    end#if
    # RE-set percentage
	@percentages_array.each_with_index do |p,i|
		js="document.getElementById(\"percentage\")[#{i}] = new Option('#{p}','#{p}'"
		js+=",true,true" if @percentage==p ### SELECTED Percent
		js+=");"
		@dlg.execute_script(js)
	end#do
    @percentage=@dlg.get_element_value("percentage")
    ### update width/height by %
    @width=(@width.to_i * @percentage.to_i / 100).to_i.to_s
    js=("document.getElementById(\"width\").setAttribute(\"value\",\""+@width+"\");")
    @dlg.execute_script(js)
    @height=(@height.to_i * @percentage.to_i / 100).to_i.to_s
    js=("document.getElementById(\"height\").setAttribute(\"value\",\""+@height+"\");")
    @dlg.execute_script(js)
  end
  
  def export_camera(p)
    @export_camera=p
    @active_camera=@dlg.get_element_value("active_camera")
    if @export_camera=="true"
        js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"true\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera_label\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
    else
        js=("document.getElementById(\"export_camera\").setAttribute(\"value\",\"false\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera\").setAttribute(\"disabled\",\"disabled\");")
        @dlg.execute_script(js)
        js=("document.getElementById(\"active_camera_label\").setAttribute(\"disabled\",\"disabled\");")
        @dlg.execute_script(js)
    end#if
  end#export_camera
  
  def set_camera()
    @active_camera=@dlg.get_element_value("active_camera")
    ### zoom to this view
    if @active_camera==@cameras_array[0]
      ###do noting
    elsif @active_camera==@cameras_array[1]
      @model.active_view.camera=@model.pages.selected_page.camera
    else
      @model.pages.selected_page=@model.pages[@active_camera]
    end#if
    ###
  end#set_camera
  
  def lens_aperture_radius(p)
    @lens_aperture_radius=p
    @lens_aperture_radius_value=@dlg.get_element_value("lens_aperture_radius_value")
    if @lens_aperture_radius=="true"
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_lens\").removeAttribute(\"disabled\");")
      #@dlg.execute_script(js)
    else
      js=("document.getElementById(\"lens_aperture_radius\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"lens_aperture_radius_value\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_lens\").setAttribute(\"disabled\",\"disabled\");")
      #@dlg.execute_script(js)
    end#if
  end#lens_aperture_radius
  
  def focal_depth(p)
    @focal_depth=p
    @focal_depth_value=@dlg.get_element_value("focal_depth_value")
    if @focal_depth=="true"
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_focal\").removeAttribute(\"disabled\");")
      #@dlg.execute_script(js)
    else
      js=("document.getElementById(\"focal_depth\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"focal_depth_value\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      #js=("document.getElementById(\"horiz_slider_focal\").setAttribute(\"disabled\",\"disabled\");")
      #@dlg.execute_script(js)
    end#if
  end#lens_aperture_radius
  
  def motion_blur(p)
    @motion_blur=p
    @motion_blur_value=@dlg.get_element_value("motion_blur_value")
    if @motion_blur=="true"
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"motion_blur\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"motion_blur_value\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
  end#lens_aperture_radius

  def export_sun_direction(p)
    @export_sun_direction=p
    @light_source=@dlg.get_element_value("light_source")
    if @export_sun_direction=="true"
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"export_sun_direction\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source_label\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"light_source\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
    end#if
  end#export_sun_direction
  
  def gpu_device0(p)
    @gpu_device0=p
    if @gpu_device0=="true"
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
  end
  def gpu_device1(p)
    @gpu_device1=p
    if @gpu_device1=="true"
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
  end
  def gpu_device2(p)
    @gpu_device2=p
    if @gpu_device2=="true"
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
  end
  def gpu_device3(p)
    @gpu_device3=p
    if @gpu_device3=="true"
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
    end#if
  end
  def gpu_devices(p)
    @gpu_devices=p
    if @gpu_devices=="true"
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"true\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
      ###
      @gpu_device0="true"
      @gpu_device1="false"
      @gpu_device2="false"
      @gpu_device3="false"
    else
      js=("document.getElementById(\"gpu_devices\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device0\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device1\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device2\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"checked\",null);")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").removeAttribute(\"checked\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"value\",\"false\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu_device3\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu0\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu1\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu2\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      js=("document.getElementById(\"gpu3\").setAttribute(\"disabled\",\"disabled\");")
      @dlg.execute_script(js)
      ###
      @gpu_device0="false"
      @gpu_device1="false"
      @gpu_device2="false"
      @gpu_device3="false"
    end#if
  end#gpu_devices
  
  def export(anim=false)
    Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Exporting OBJ & MTL Files...")))
    self.getIDvalues()
    self.setIDvalues()### set variables to these values too...
    ### check for validity
    if not @project_path or @project_path==""
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Project Path is not valid.")))
      return nil
    end#if
    special=false
    special=true if @pc and @project_path.length != @project_path.unpack("C*").pack("U*").length
    if special
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Project Path is not valid."))+"\n"+(db("NO accented or special-characters allowed.")))
      return nil
    end#if
    if not fileExist?(@project_path)
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Project Path is not valid.")))
      return nil
    end#if
    if @project_name=~/ /
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Project Name is not valid.")))
      return nil
    end#if
    ###
    if anim==false
      self.run_exporter()
    else
      ### zoom to start_frame to set OBJ/MTL
      @model.pages.selected_page=@model.pages[@start_frame]
      ### 6=YES 7=NO
      yn=UI.messagebox((db("Octane Render:"))+"\n\n"+(db("Animate just the Camera?"))+"\n\n'"+(db("YES"))+"' \t"+(db("Makes ONE OBJ/MTL file, based on"))+" '"+@start_frame+"'\n\t"+(db("Use this for walkthroughs, flythroughs, orbits etc..."))+"\n\n'"+(db("NO"))+"' \t"+(db("Makes MULTIPLE OBJ/MTL files, that's one for every frame!"))+"\n\t"+(db("Use this only if you want to 'animate' objects, visibility etc...")),MB_YESNOCANCEL,"")
      if yn==6 ### use ONE OBJ/MTL file set
        @yn=true
        self.run_exporter("-")
        sleep(0.2)
        return true
      elsif yn==7 ### multiple OBJ/MTLs
        @yn=false
        ###
        startf=@start_frame
        endf=@end_frame
        step=@step.to_i
        pages=@model.pages
        cameras=[]; pages.each{|p|cameras << p.camera}
        ###
        scenes=@scenes
        s_indx=scenes.index(startf)
        e_indx=scenes.index(endf)
        anim_scenes=scenes[s_indx..e_indx]
        ### adjust array for step
        step_scenes=[]
        0.step(anim_scenes.length-1, step){|i|step_scenes << anim_scenes[i]}
        anim_scenes=step_scenes
        ###
        selpage=@model.pages.selected_page
        indx=0
        anim_scenes.each{|s|
          @model.pages.selected_page=@model.pages[s]
          self.run_exporter(anim=indx.to_s)
          indx+=1
          sleep(0.2)
        }
        @model.pages.selected_page=selpage
        return true
      else ### cancelled
        @yn=true
        self.set_button(false)
        js=("document.getElementById(\"render_animation_button\").removeAttribute(\"disabled\");")
        @dlg.execute_script(js)
        return nil
      end#if
    end#if
    ###
  end#export
  
  def render()
    self.export()
    ###
	Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Render...")))
    ### make ocs if not there
    prp=@project_path.tr("/", "\\")
    prn=@project_name
    ocs=prp+"\\"+prn+".ocs" ###
    obj=prp+"\\"+prn+".obj" ###
    obn=prn+".obj" ###
    ### set variables for command-line
    cline=""
    if @pc
      cline=cline+"@echo off\n"
      drive=@project_path.split(":")[0]+":\n"
      cline=cline+drive
    end#if
    cline=cline+"title Render Cmd\n"
    ###
    orb=@octane_render_binary
    #orb="cmd /u /k "+orb if @pc ###?????????????????????????????
    orb=orb+"/Contents/MacOS/octane" if not @pc
    cline=cline+"@" if @pc
    cline=cline+"\""+orb+"\""
    ###
    prp=@project_path.tr("/", "\\")
    prn=@project_name
    ocs=prp+"\\"+prn+".ocs"
    ona=@obj_name
    ofi=@obj_filepath.tr("/", "\\")
    ###
    ### add in 'new' if not fileExist?
    if not fileExist?(ocs)
      cline=cline+" -n \""+ocs+"\""
      cline=cline+" -l \""+ofi+"\""
    end#if
    ###
    ###cline=cline+" -s 4096 "### so continues to render to very high till stopped
    cline=cline+" -m \""+ona+"\""
    cline=cline+" -r \""+ofi+"\"" if fileExist?(ocs)
    ###
    if @resolution=="true"
      wi=@width
      he=@height
      cline=cline+" --film-width " +wi.to_s
      cline=cline+" --film-height "+he.to_s
    else
      wi=nil
      he=nil
    end#if
    ### active camera
  if @export_camera=="true"
    cams=@cameras_array
    acc=@model.active_view.camera
    pages=@model.pages
    case @active_camera
      when (db("from <Current View>"))
        acc=@model.active_view.camera
      when (db("from <Current Scene>"))
        acc=@model.pages.selected_page.camera
      else
        pages.each{|p|acc=p.camera if p.name==@active_camera}
    end#case    
    posa=acc.eye.to_a
    pos=[0,0,0]
    pos.x=posa.x.to_m.to_f
    pos.y=posa.z.to_m.to_f      ###
    pos.z=posa.y.to_m.to_f * -1 ###
    targeta=acc.target.to_a
    target=[0,0,0]
    target.x=targeta.x.to_m.to_f
    target.y=targeta.z.to_m.to_f        ###
    target.z=targeta.y.to_m.to_f * -1   ###
    upa=acc.up.normalize.to_a
    up=[0,0,0]
    up.x=upa.x.to_f
    up.y=upa.z.to_f        ###
    up.z=upa.y.to_f * -1   ###
    ###
    cline=cline+" --cam-pos-x "+pos.x.to_s
    cline=cline+" --cam-pos-y "+pos.y.to_s
    cline=cline+" --cam-pos-z "+pos.z.to_s
    ###
    cline=cline+" --cam-target-x "+target.x.to_s
    cline=cline+" --cam-target-y "+target.y.to_s
    cline=cline+" --cam-target-z "+target.z.to_s
    ###
    cline=cline+" --cam-up-x "+up.x.to_s
    cline=cline+" --cam-up-y "+up.y.to_s
    cline=cline+" --cam-up-z "+up.z.to_s
    ###
    fol=acc.focal_length
    fovV=acc.fov
    wid=@width.to_f
    hei=@height.to_f
    if acc.image_width != 0
      wid=acc.image_width
      if acc.aspect_ratio != 0
        aro=acc.aspect_ratio
      else
        aro=wid/hei
      end#if
      hei=wid/aro
    end#if
    h=2*fol*Math::tan(fovV.degrees/2)
    w=h*wid/hei
    fov=(2*Math::atan(w/(2*fol))).radians
    cline=cline+" --cam-fov "+fov.to_s
    ###
  end#if export_camera
    ###
    if @lens_aperture_radius=="true"
      apr=@lens_aperture_radius_value
      cline=cline+" --cam-aperture "+apr.to_s
    else
      apr=nil
    end#if
    if @focal_depth=="true"
      foc=@focal_depth_value
      cline=cline+" --cam-focaldepth "+foc.to_s
    end#if
    ###
    if @motion_blur=="true"
      mob=@motion_blur_value
    else
      mob=""
    end#if
    mbc=nil
    case mob
      when (db("Next"))
        if @active_camera==(db("from <Current View>"))
          mbc=nil
        elsif @active_camera==(db("from <Current Scene>"))
          pages=@model.pages.to_a
          cac=@model.pages.selected_page
          cur=pages.index(cac)
          nxt=cur+1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[0].camera
          end#if
        else
          pages=@model.pages.to_a
          cac=pages[0]
          pages.each{|p|cac=p if p.name==@active_camera}
          cur=pages.index(cac)
          nxt=cur+1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[0].camera
          end#if
        end#if
      when (db("Previous"))
        if @active_camera==(db("from <Current View>"))
          mbc=nil
        elsif @active_camera==(db("from <Current Scene>"))
          pages=@model.pages.to_a
          cac=@model.pages.selected_page
          cur=pages.index(cac)
          nxt=cur-1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[-1].camera
          end#if
        else
          pages=@model.pages.to_a
          cac=pages[0]
          pages.each{|p|cac=p if p.name==@active_camera}
          cur=pages.index(cac)
          nxt=cur-1
          mbc=pages[nxt]
          if mbc
            mbc=mbc.camera
          else
            mbc=pages[-1].camera
          end#if
        end#if
      else
        mbc=nil
    end#case
    if mbc
      mposa=mbc.eye.to_a
      mpos=mposa
      mpos.x=mposa.x.to_m.to_f
      mpos.y=mposa.z.to_m.to_f
      mpos.z=mposa.y.to_m.to_f * -1
      mtargeta=mbc.target.to_a
      mtarget=mtargeta
      mtarget.x=mtargeta.x.to_m.to_f
      mtarget.y=mtargeta.z.to_m.to_f
      mtarget.z=mtargeta.y.to_m.to_f * -1
      mupa=mbc.up.normalize.to_a
      mup=mupa
      mup.x=mupa.x.to_f
      mup.y=mupa.z.to_f
      mup.z=mupa.y.to_f * -1
      ###
      cline=cline+" --cam-motion-pos-x "+mpos.x.to_s
      cline=cline+" --cam-motion-pos-y "+mpos.y.to_s
      cline=cline+" --cam-motion-pos-z "+mpos.z.to_s
      ###
      cline=cline+" --cam-motion-target-x "+mtarget.x.to_s
      cline=cline+" --cam-motion-target-y "+mtarget.y.to_s
      cline=cline+" --cam-motion-target-z "+mtarget.z.to_s
      ###
      cline=cline+" --cam-motion-up-x "+mup.x.to_s
      cline=cline+" --cam-motion-up-y "+mup.y.to_s
      cline=cline+" --cam-motion-up-z "+mup.z.to_s
      ###
    end#if
    ### sun etc ?
    if @export_sun_direction=="true"
      pages=@model.pages.to_a
      case @light_source
        when (db("Sun from <Current View>"))
          sunv=@model.shadow_info["SunDirection"]
        when (db("Sun from <Current Scene>"))
          sunv=@model.pages.selected_page.shadow_info["SunDirection"]
        else
          spa=nil
          pages.each{|p|spa=p if p.name==@light_source}
          sunv=spa.shadow_info["SunDirection"]
      end#case
      suna=sunv.normalize.to_a
      sun=[0,0,0]
      sun.x=suna.x.to_f
      sun.y=suna.z.to_f
      sun.z=suna.y.to_f * -1
      cline=cline+" --daylight-sundir-x "+sun.x.to_s
      cline=cline+" --daylight-sundir-y "+sun.y.to_s
      cline=cline+" --daylight-sundir-z "+sun.z.to_s
    end#if
    ### GPUs ?
    if @gpu_devices=="true"
      if @gpu_device0=="true"
        cline=cline+" -g 0"
      end#if
      if @gpu_device1=="true"
        cline=cline+" -g 1"
      end#if
      if @gpu_device2=="true"
        cline=cline+" -g 2"
      end#if
      if @gpu_device3=="true"
        cline=cline+" -g 3"
      end#if
    end#if
    ###
    if not fileExist?(ocs)
      cline=cline+"\n"
    else
      cline=cline+" \""+ocs+"\"\n"
    end#if
    ###
    cline=cline+"@exit\n" if @pc
    ############################ render
    if @pc
      ### make cmd file in TEMP
      #begin#1
        #temp=ENV["TEMP"]
        #if not fileExist?(temp)
          #dirMkdir(temp)
        #end#if
      #rescue#1
        #begin#2
          #temp="C:\\Temp"
          #if not fileExist?(temp)
            #dirMkdir(temp)
          #end#if
          #temp=@project_path.tr("\\", "/")
        #rescue#2
          begin#3
            temp=@project_path.tr("\\", "/")
            if not fileExist?(temp)
              dirMkdir(temp)
            end#if
          rescue#3
            UI.messagebox("RENDER-TEMP-CMD !?!?")
            return nil
          end#3
        #end#2
      #end#1
      ###
      cmd_path=fileJoin(temp, "octane.cmd")
      cmd=fileNew(cmd_path, "w")
      cmd.puts(cline)
      cmd.close
      ### make start.cmd
      start_path=fileJoin(temp, "start.cmd")
      cmd=fileNew(start_path, "w")
      drive=@project_path.split(":")[0]+":\n"
      cmd.puts(drive)
      cmd.puts(("cd ")+(temp.tr("/", "\\"))+("\\\nstart /MIN octane.cmd\nexit\n"))
      cmd.puts("del \""+(start_path.tr("/", "\\"))+"\"")
      cmd.close
      sleep(0.5)###wait before continuing
      ### run cmd file minimized
      UI.openURL(start_path)
      sleep(3.0);fileDelete(start_path) if fileExist?(start_path)
    else ### it's a Mac
      cline="#!/bin/bash\n"+cline ### makes it execute in a shell ???
      ### ???
      cline.tr!("\\", "/")### force unix friendly separators
      cline=cline.gsub(/\"/){'\\"'}### escape " as \" IS NEEDED - SO DO ALL
      ### NOW we must un-escape all \"app\" to "app" ...
      cline=cline.gsub(/\\\"#{orb.tr("\\", "/")}\\\"/, "\"#{orb.tr("\\", "/")}\"")
      cline=cline.gsub(/^@/){''}### removing leading @'s IF ANY ???
      ### change 'title' to suit Mac
      cline=cline.sub(/^title Render Cmd/){'echo -ne "\033]0;Render Cmd\007"'}###########
      ### fix echo off to suit Mac
      cline=cline.sub(/^echo off\n/, '')
      ###
      cmd_path=fileJoin(@project_path,"octane.command")
      cmd=fileNew(cmd_path, "w")
      cmd.puts(cline)
      cmd.close
      fileChmod(0777, cmd_path)
      ### run cmd file
      UI.openURL("file:///"+cmd_path)
      ###
    end#if
    @cmd=cmd_path
    ###
  end#render
  
  def select_image_output()
    Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Select Image Output Folder...")))
    ### browse for folder
    @image_output=@dlg.get_element_value("image_output")
    ###
    if fileExist?(@project_path)
      self.startfb(@project_path, "image")
    else
      self.startfb(dirPwd, "image")
    end#if
    ###
    Sketchup.set_status_text(db("Octane Render:"))
    ###
  end#select_image_output
  
  def update_image()
    image_output = @octane_render_folder_browser_result
    #image_output=UI.openpanel("Choose the 'Image Output' File [.PNG] or Any File to Select its Folder...")
    Sketchup.set_status_text(db("Octane Render:"))
    if not image_output
      return nil
    end#if
    special=false
    special=true if @pc and image_output.length != image_output.unpack("C*").pack("U*").length
    if special
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Octane Render Binary'!"))+"\n"+(db("NO accented or special-characters allowed.")))
      self.deactivate()
      return nil
    end#if
    image_out=fileJoin(image_output, @image_name)
    image_out.tr!("/", "\\")if @pc
    @image_output=image_out
    @model.set_attribute("Octane", "image_output", @image_output)
    ###
    image_out=@image_output.gsub(/\\/){'\\\\'} if @pc
    js=("document.getElementById(\"image_output\").setAttribute(\"value\",\""+image_out+"\");")
    @dlg.execute_script(js)
  end#update_image
  
  def update_pixs()
    @pixel_samples=@dlg.get_element_value("pixel_samples")
    pixsa=@pixel_samples
    if pixsa==(db("Custom"))
      pixs=@pixs
    else
      pixs=pixsa.split("(")[1].to_i.to_s
    end#if    
    js=("document.getElementById(\"pixs\").setAttribute(\"value\",\""+pixs+"\");")
    @dlg.execute_script(js)
    @pixs=@dlg.get_element_value("pixs")
  end#update_pixs
  
  def update_pixs_txt()
    pixs=@dlg.get_element_value("pixs")
    pix=pixs.to_i.to_s
    ### update list too
    tester=(db("Custom"))
    pixel_samples_array=@pixel_samples_array
    (pixel_samples_array-[tester]).each{|txt|
      if pix == txt.split("(")[1].to_i.to_s
        tester=txt
        break
      end#if
    }
    pixel_samples_array.each_with_index do |p,i|
    js="document.getElementById(\"pixel_samples\")[#{i}] = new Option('#{p}','#{p}'"
	js+=",true,true" if p==tester ### SELECTED pixel_samples
	js+=");"
	@dlg.execute_script(js)
    @pixel_samples=@dlg.get_element_value("pixel_samples")
    end#if
    js=("document.getElementById(\"pixs\").setAttribute(\"value\",\""+pix+"\");")
    @dlg.execute_script(js)
    @pixs=@dlg.get_element_value("pixs")
  end#update_pixs_txt
  
  def update_start(pop=false)
    @start_frame=@dlg.get_element_value("start_frame")if not pop
    if not @scenes.include?(@start_frame)
      if @scenes[1]
        @start_frame=@scenes[0]
      else
        @start_frame=nil
        return nil
      end#if
    end#if
    start_frame_array=@scenes[0..-2] ### not the last one
	start_frame_array.each_with_index do |p,i|
		js="document.getElementById(\"start_frame\")[#{i}] = new Option('#{p}','#{p}'"
        js+=",true,true" if @start_frame==p ### SELECTED start_frame
		js+=");"
		@dlg.execute_script(js)
	end #if start_frame_array[0]
    #do
    @start_frame=@dlg.get_element_value("start_frame")
  end#update_start
  
  def update_end(pop=false)
    @end_frame=@dlg.get_element_value("end_frame")if not pop
    if not @scenes.include?(@end_frame)
      if @scenes[1]
        @end_frame=@scenes[1]
      else
        @end_frame=nil
        return nil
      end#if
    end#if
    end_frame_array=@scenes[1..-1] ### not the first one
	end_frame_array.each_with_index do |p,i|
		js="document.getElementById(\"end_frame\")[#{i}] = new Option('#{p}','#{p}'"
        js+=",true,true" if @end_frame==p ### SELECTED end_frame
		js+=");"
		@dlg.execute_script(js)
	end #if end_frame_array[0]
    #do
    @end_frame=@dlg.get_element_value("end_frame")
  end#update_end
  
  def update_step(pop=false)
    if not @step or @step==""
      @step="1"
    elsif not pop
      begin
        @step=@dlg.get_element_value("step")
      rescue
        @step="1"
      end
    end#if
    ### calculate step list
    step_array=[]
    if @scenes[1]
      @scenes[1..-1].length.times{|i|step_array << (i+1).to_s}
    end#if
    if not step_array.include?(@step) or not step_array[0]
      @step="1"
    end#if
    step_array.each_with_index do |p,i|
	  js="document.getElementById(\"step\")[#{i}] = new Option('#{p}','#{p}'"
      js+=",true,true" if @step==p ### SELECTED step
	  js+=");"
	  @dlg.execute_script(js)
	end#do
    @step=@dlg.get_element_value("step")
  end#update_step
  
  def get_scenes()### only 'anim' scenes
    @all_pages=@model.pages.to_a
    @scenes=[]; @all_pages.each{|p|@scenes<< p.name if p.name==p.label}
  end#get_scenes
  
  
  def render_animation()
    if not @start_frame or @start_frame=="" or not @scenes.flatten.include?(@start_frame)
      UI.messagebox((db("Octane Render:"))+"\n"+(db("There are no Scenes to animate!")))
      return nil
    end#if
    self.set_button(true)
    js=("document.getElementById(\"render_animation_button\").setAttribute(\"disabled\",\"disabled\");")
    @dlg.execute_script(js)
    ###
    anim=true
    return nil if not self.export(anim) ### also gets IDs
    ###
    ### check if multi-PBJ
    if @yn ### single OBJ
      inc=""
    else ### multi OBJ
      inc="0"
    end#if
    ###
    Sketchup.set_status_text((db("Octane Render:"))+" "+(db("To Abort - Close the Separate 'Animation Cmd' Window...")))
    ###
    startf=@start_frame
    endf=@end_frame
    step=@step.to_i
    ###
    opu=@image_output
    opudiry=fileDirname(opu)
    opuextn=fileExtname(opu)
    opuname=fileBasename(opu, opuextn)
    opu=fileJoin(opudiry, opuname)
    opu.tr!("/", "\\")
    ###
    pages=@model.pages
    cameras=[]; pages.each{|p|cameras << p.camera}
    ###
    scenes=@scenes
    s_indx=scenes.index(startf)
    e_indx=scenes.index(endf)
    anim_scenes=scenes[s_indx..e_indx]
    ### adjust array for step
    step_scenes=[]
    0.step(anim_scenes.length-1, step){|i|step_scenes << anim_scenes[i]}
    anim_scenes=step_scenes
    ### 
    ### make we FIRDT frame - making ocs if not fileExist?
    scene_name=anim_scenes[0]
    scene=nil
    @model.pages.each{|p|
      if p.name==scene_name
        scene=p
        break
      end#if
    }
    return nil if not scene
    ###
    ### set frame >>>>>>>>>>>>>>>>>>>
    ### set variables for command-line
    cline=""
    if @pc
      cline=cline+"@echo off\n"
      drive=@project_path.split(":")[0]+":\n"
      cline=cline+drive
    end#if
    cline=cline+"title Animation Cmd\n"
    ###
    orb=@octane_render_binary
    orb=orb+"/Contents/MacOS/octane" if not @pc
    cline=cline+"@" if @pc
    cline=cline+"\""+orb+"\" -e -q"
    ###
    prp=@project_path.tr("/", "\\")
    prn=@project_name
    ocs=prp+"\\"+prn+".ocs"
    oob=prn+".obj" ### ORIGINAL BASE OBJ###
    obj=prp+"\\"+prn+".obj"
    ona=oob
    ofi=fileJoin(@project_path.tr("/", "\\"), prn+'_anim'+inc+'.obj')
    ### add in 'new' if not fileExist?
    if not fileExist?(ocs)
      cline=cline+" -n \""+ocs+"\""
      cline=cline+" -l \""+obj+"\""
    end#if    ###
    sam=@pixs
    cline=cline+" -s "+sam
    cline=cline+" -m \""+ona+"\""
    cline=cline+" -r \""+ofi+"\"" if fileExist?(ocs)
    ###
    ### here we add the opu etc
    opux=opu+opuextn
    ###
    cline=cline+" -o \""+opux+"\""
    ###
    if @resolution=="true"
      wi=@width
      he=@height
      cline=cline+" --film-width " +wi.to_s
      cline=cline+" --film-height "+he.to_s
    else
      wi=nil
      he=nil
    end#if
    ###
    ### ALWAYS export the camera !
  #if @export_camera=="true"
    ### get camera
    acc=scene.camera
    posa=acc.eye.to_a
    pos=[0,0,0]
    pos.x=posa.x.to_m.to_f
    pos.y=posa.z.to_m.to_f
    pos.z=posa.y.to_m.to_f * -1
    targeta=acc.target.to_a
    target=[0,0,]
    target.x=targeta.x.to_m.to_f
    target.y=targeta.z.to_m.to_f
    target.z=targeta.y.to_m.to_f * -1
    upa=acc.up.normalize.to_a
    up=[0,0,0]
    up.x=upa.x.to_f
    up.y=upa.z.to_f
    up.z=upa.y.to_f * -1
    ###
    cline=cline+" --cam-pos-x "+pos.x.to_s
    cline=cline+" --cam-pos-y "+pos.y.to_s
    cline=cline+" --cam-pos-z "+pos.z.to_s
    ###
    cline=cline+" --cam-target-x "+target.x.to_s
    cline=cline+" --cam-target-y "+target.y.to_s
    cline=cline+" --cam-target-z "+target.z.to_s
    ###
    cline=cline+" --cam-up-x "+up.x.to_s
    cline=cline+" --cam-up-y "+up.y.to_s
    cline=cline+" --cam-up-z "+up.z.to_s
    ###
    fol=acc.focal_length
    fovV=acc.fov
    wid=@width.to_f
    hei=@height.to_f
    if acc.image_width != 0
      wid=acc.image_width
      if acc.aspect_ratio != 0
        aro=acc.aspect_ratio
      else
        aro=wid/hei
      end#if
      hei=wid/aro
    end#if
    h=2*fol*Math::tan(fovV.degrees/2)
    w=h*wid/hei
    fov=(2*Math::atan(w/(2*fol))).radians
    cline=cline+" --cam-fov "+fov.to_s
  #end#if export_camera
    ###
    if @lens_aperture_radius=="true"
      apr=@lens_aperture_radius_value
      cline=cline+" --cam-aperture "+apr.to_s
    else
      apr=nil
    end#if
    if @focal_depth=="true"
      foc=@focal_depth_value
      cline=cline+" --cam-focaldepth "+foc.to_s
    end#if
    ###
    if @motion_blur=="true"
      mob=@motion_blur_value
    else
      mob=""
    end#if
    mbc=nil
    case mob
      when (db("Next"))
        mbc=nil
        scene_indx=scenes.index(scene)
        if scene_indx
          mb_indx=scene_indx+1
          mbc=scenes[mb_indx]
        end#if
      when (db("Previous"))
        mbc=nil
        scene_indx=scenes.index(scene)
        if scene_indx
          mb_indx=scene_indx-1
          mbc=scenes[mb_indx]
        end#if
      else
        mbc=nil
    end#case
    if mbc
      mposa=mbc.eye.to_a
      mpos=[0,0,0]
      mpos.x=mposa.x.to_m.to_f
      mpos.y=mposa.z.to_m.to_f
      mpos.z=mposa.y.to_m.to_f * -1
      mtargeta=mbc.target.to_a
      mtarget=[0,0,0]
      mtarget.x=mtargeta.x.to_m.to_f
      mtarget.y=mtargeta.z.to_m.to_f
      mtarget.z=mtargeta.y.to_m.to_f * -1
      mupa=mbc.up.normalize.to_a
      mup=[0,0,0]
      mup.x=mupa.x.to_f
      mup.y=mupa.z.to_f
      mup.z=mupa.y.to_f * -1
      ###
      cline=cline+" --cam-motion-pos-x "+mpos.x.to_s
      cline=cline+" --cam-motion-pos-y "+mpos.y.to_s
      cline=cline+" --cam-motion-pos-z "+mpos.z.to_s
      ###
      cline=cline+" --cam-motion-target-x "+mtarget.x.to_s
      cline=cline+" --cam-motion-target-y "+mtarget.y.to_s
      cline=cline+" --cam-motion-target-z "+mtarget.z.to_s
      ###
      cline=cline+" --cam-motion-up-x "+mup.x.to_s
      cline=cline+" --cam-motion-up-y "+mup.y.to_s
      cline=cline+" --cam-motion-up-z "+mup.z.to_s
      ###
    end#if
    ### sun etc ?
    if @export_sun_direction=="true"
      sunv=scene.shadow_info["SunDirection"]
      suna=sunv.normalize.to_a
      sun=[0,0,0]
      sun.x=suna.x.to_f
      sun.y=suna.z.to_f
      sun.z=suna.y.to_f * -1
      cline=cline+" --daylight-sundir-x "+sun.x.to_s
      cline=cline+" --daylight-sundir-y "+sun.y.to_s
      cline=cline+" --daylight-sundir-z "+sun.z.to_s
    end#if
    ### GPUs ?
    if @gpu_devices=="true"
      if @gpu_device0=="true"
        cline=cline+" -g 0"
      end#if
      if @gpu_device1=="true"
        cline=cline+" -g 1"
      end#if
      if @gpu_device2=="true"
        cline=cline+" -g 2"
      end#if
      if @gpu_device3=="true"
        cline=cline+" -g 3"
      end#if
    end#if
    if not fileExist?(ocs)
      cline=cline+"\n"
    else
      cline=cline+" \""+ocs+"\"\n"
    end#if
    ###
    opu.next! ### increments the img00001 > img00002 etc
    inx=1
    ### make rest of scenes
  anim_scenes[1..-1].each{|scene_name|
    if @yn
      inc=""
    else
      inc=inx.to_s
    end#if
    scene=nil
    @model.pages.each{|p|
      if p.name==scene_name
        scene=p
        break
      end#if
    }
    next if not scene
    ### set variables for command-line
    ###
    orb=@octane_render_binary
    orb=orb+"/Contents/MacOS/octane" if not @pc
    cline=cline+"@" if @pc
    cline=cline+"\""+orb+"\" -e -q"
    ###
    ### here we add the opu etc
    opux=opu+opuextn
    ###
    cline=cline+" -o \""+opux+"\""
    ###
    sam=@pixs
    cline=cline+" -s "+sam
    ###
    ona=oob ###@obj_name
    cline=cline+" -m \""+ona+"\"" ### OOB NOT @obj_name
    ###
    ofi=fileJoin(@project_path.tr("/", "\\"), prn+'_anim'+inc+'.obj')
    #cline=cline+" -l \""+ofi+"\"" ### no need to link in anim obj ?
    cline=cline+" -r \""+ofi+"\"" if fileExist?(ocs)
    ###
    if @resolution=="true"
      wi=@width
      he=@height
      cline=cline+" --film-width " +wi.to_s
      cline=cline+" --film-height "+he.to_s
    else
      wi=nil
      he=nil
    end#if
    ###
    ### ALWAYS export the camera !
  #if @export_camera=="true"
    ### get camera
    acc=scene.camera
    posa=acc.eye.to_a
    pos=[0,0,0]
    pos.x=posa.x.to_m.to_f
    pos.y=posa.z.to_m.to_f
    pos.z=posa.y.to_m.to_f * -1
    targeta=acc.target.to_a
    target=[0,0,]
    target.x=targeta.x.to_m.to_f
    target.y=targeta.z.to_m.to_f
    target.z=targeta.y.to_m.to_f * -1
    upa=acc.up.normalize.to_a
    up=[0,0,0]
    up.x=upa.x.to_f
    up.y=upa.z.to_f
    up.z=upa.y.to_f * -1
    ###
    cline=cline+" --cam-pos-x "+pos.x.to_s
    cline=cline+" --cam-pos-y "+pos.y.to_s
    cline=cline+" --cam-pos-z "+pos.z.to_s
    ###
    cline=cline+" --cam-target-x "+target.x.to_s
    cline=cline+" --cam-target-y "+target.y.to_s
    cline=cline+" --cam-target-z "+target.z.to_s
    ###
    cline=cline+" --cam-up-x "+up.x.to_s
    cline=cline+" --cam-up-y "+up.y.to_s
    cline=cline+" --cam-up-z "+up.z.to_s
    ###
    fol=acc.focal_length
    fovV=acc.fov
    wid=@width.to_f
    hei=@height.to_f
    if acc.image_width != 0
      wid=acc.image_width
      if acc.aspect_ratio != 0
        aro=acc.aspect_ratio
      else
        aro=wid/hei
      end#if
      hei=wid/aro
    end#if
    h=2*fol*Math::tan(fovV.degrees/2)
    w=h*wid/hei
    fov=(2*Math::atan(w/(2*fol))).radians
    cline=cline+" --cam-fov "+fov.to_s
  #end#if export_camera
    ###
    if @lens_aperture_radius=="true"
      apr=@lens_aperture_radius_value
      cline=cline+" --cam-aperture "+apr.to_s
    else
      apr=nil
    end#if
    if @focal_depth=="true"
      foc=@focal_depth_value
      cline=cline+" --cam-focaldepth "+foc.to_s
    end#if
    ###
    if @motion_blur=="true"
      mob=@motion_blur_value
    else
      mob=""
    end#if
    mbc=nil
    case mob
      when (db("Next"))
        mbc=nil
        scene_indx=scenes.index(scene)
        mb_indx=scene_indx+1
        mbc=scenes[mb_indx]
      when (db("Previous"))
        mbc=nil
        scene_indx=scenes.index(scene)
        mb_indx=scene_indx-1
        mbc=scenes[mb_indx]
      else
        mbc=nil
    end#case
    if mbc
      mposa=mbc.eye.to_a
      mpos=[0,0,0]
      mpos.x=mposa.x.to_m.to_f
      mpos.y=mposa.z.to_m.to_f
      mpos.z=mposa.y.to_m.to_f * -1
      mtargeta=mbc.target.to_a
      mtarget=[0,0,0]
      mtarget.x=mtargeta.x.to_m.to_f
      mtarget.y=mtargeta.z.to_m.to_f
      mtarget.z=mtargeta.y.to_m.to_f * -1
      mupa=mbc.up.normalize.to_a
      mup=[0,0,0]
      mup.x=mupa.x.to_f
      mup.y=mupa.z.to_f
      mup.z=mupa.y.to_f * -1
      ###
      cline=cline+" --cam-motion-pos-x "+mpos.x.to_s
      cline=cline+" --cam-motion-pos-y "+mpos.y.to_s
      cline=cline+" --cam-motion-pos-z "+mpos.z.to_s
      ###
      cline=cline+" --cam-motion-target-x "+mtarget.x.to_s
      cline=cline+" --cam-motion-target-y "+mtarget.y.to_s
      cline=cline+" --cam-motion-target-z "+mtarget.z.to_s
      ###
      cline=cline+" --cam-motion-up-x "+mup.x.to_s
      cline=cline+" --cam-motion-up-y "+mup.y.to_s
      cline=cline+" --cam-motion-up-z "+mup.z.to_s
      ###
    end#if
    ### sun etc ?
    if @export_sun_direction=="true"
      sunv=scene.shadow_info["SunDirection"]
      suna=sunv.normalize.to_a
      sun=[0,0,0]
      sun.x=suna.x.to_f
      sun.y=suna.z.to_f
      sun.z=suna.y.to_f * -1
      cline=cline+" --daylight-sundir-x "+sun.x.to_s
      cline=cline+" --daylight-sundir-y "+sun.y.to_s
      cline=cline+" --daylight-sundir-z "+sun.z.to_s
    end#if
    ### GPUs ?
    if @gpu_devices=="true"
      if @gpu_device0=="true"
        cline=cline+" -g 0"
      end#if
      if @gpu_device1=="true"
        cline=cline+" -g 1"
      end#if
      if @gpu_device2=="true"
        cline=cline+" -g 2"
      end#if
      if @gpu_device3=="true"
        cline=cline+" -g 3"
      end#if
    end#if
    cline=cline+" \""+ocs+"\"\n"
    ###
    opu.next! ### increments the img00001 > img00002 etc
    ###
    inx=inx+=1### increments OBJs
    ###
  }#anim_scenes
    ###
    cline=cline+"@exit\n"
    ###
    ############## anim
    if @pc
      ### make cmd file in TEMP
      #begin#1
        #temp=ENV["TEMP"]
        #if not fileExist?(temp)
          #dirMkdir(temp)
        #end#if
      #rescue#1
        #begin#2
          #temp="C:\\Temp"
          #if not fileExist?(temp)
            #dirMkdir(temp)
          #end#if
          #temp=@project_path.tr("\\", "/")
        #rescue#2
          begin#3
            temp=@project_path.tr("\\", "/")
            if not fileExist?(temp)
              dirMkdir(temp)
            end#if
          rescue#3
            UI.messagebox("ANIM-TEMP-CMD !?!?")
            return nil
          end#3
        #end#2
      #end#1
      ###
      cmd_path=fileJoin(temp, "octane_anim.cmd")
      cmd=fileNew(cmd_path, "w")
      cmd.puts(cline)
      cmd.close
      ### make start_anim.cmd
      start_path=fileJoin(temp, "start.cmd")
      cmd=fileNew(start_path, "w")
      drive=@project_path.split(":")[0]+":\n"
      cmd.puts(drive)
      cmd.puts(("cd ")+(temp.tr("/", "\\"))+("\\\nstart /MIN octane_anim.cmd\nexit\n"))
      cmd.puts("del \""+(start_path.tr("/", "\\"))+"\"")
      cmd.close
      sleep(0.5)###wait before continuing
      ### run acmd file minimized
      UI.openURL(start_path)
      sleep(3.0);fileDelete(start_path) if fileExist?(start_path)
    else ### it's a Mac
      cline="#!/bin/bash\n"+cline ### makes it execute in a shell ???
      ### ???
      cline.tr!("\\", "/")### force unix friendly separators
      cline=cline.gsub(/\"/){'\\"'}### escape " as \" IS NEEDED - SO DO ALL
      ### NOW we must un-escape all \"app\" to "app" ...
      cline=cline.gsub(/\\\"#{orb.tr("\\", "/")}\\\"/, "\"#{orb.tr("\\", "/")}\"")
      cline=cline.gsub(/^@/){''}### removing leading @'s IF ANY ???
      ### change 'title' to suit Mac
      cline=cline.sub(/^title Animation Cmd/){'echo -ne "\033]0;Animation Cmd\007"'}###########
      ### fix echo off to suit Mac
      cline=cline.sub(/^echo off\n/, '')
      ###
      cmd_path="file:///"+@project_path.tr("\\", "/")+"/octane_anim.command"
      cmd=fileNew(cmd_path, "w")
      cmd.puts(cline)
      cmd.close
      fileChmod(0777, cmd_path)
      ### run cmd file
      UI.openURL("file:///"+cmd_path)
      ###
    end#if
    @acmd=cmd_path
    ###
    ### we must wait for frames to be made... or cancel HOW?
    ### = close cmd/command window = end
    if @pc
      sleep(2)
      procs=self.processes_pc()
      while procs.include?("cmd.exe")
        sleep(1)
        procs=self.processes_pc()
      end#wait till cmd closed
      self.set_button(false)
      js=("document.getElementById(\"render_animation_button\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    else ### Mac
      sleep(2)
      procs=self.processes_mac()
      while procs.include?("command")
        sleep(1)
        procs=self.processes_mac()
      end#wait till cmd closed
      self.set_button(false)
      js=("document.getElementById(\"render_animation_button\").removeAttribute(\"disabled\");")
      @dlg.execute_script(js)
    end#if  
    ###
    Sketchup.set_status_text(db("Octane Render:"))
    ###
  end#render_animation
  
  def processes_pc()
    return if not @pc
    require 'win32ole.so'
    procs=WIN32OLE.connect("winmgmts:\\\\.")
    ps=[]
    procs.InstancesOf("win32_process").each{|p|ps << p.name.to_s.downcase}
    return ps
  end#processes_pc
  
  def processes_mac()
    #puts "Mac process list."
    return [] ### to add code here
  end#processes_mac
  
  def set_button(ren=false)
    if ren
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"value\",\""+(db("Rendering Animation, Please Wait..."))+"\");")
      @dlg.execute_script(js)
    else
      js=("document.getElementById(\"render_animation_button\").setAttribute(\"value\",\""+(db("Render Animation"))+"\");")
      @dlg.execute_script(js)
    end#if
  end#set_button
  
  
  def help()
    help="OctaneHelp.pdf"
    locale=Sketchup.get_locale.upcase
    helpLocale="OctaneHelp"+locale+".pdf"
    help_path=fileJoin(fileDirname(__FILE__), help)
    helpLocale_path=fileJoin(fileDirname(__FILE__), helpLocale)
    if fileExist?(helpLocale_path)
      UI.openURL(helpLocale_path) if @pc
      UI.openURL("file:///"+helpLocale_path) if not @pc
    elsif fileExist?(help_path)
      UI.openURL(help_path) if @pc
      UI.openURL("file:///"+help_path) if not @pc
    else
      UI.messagebox((db("Octane Render:"))+"\n"+(db("Help file not found"))+"\n["+help_path+"]")
    end#if
  end#help
  
  def getIDvalues()
    @values={} ### hash to hold id values - ALL read in as strings - even if boolean
    @@idset.each{|id|@values[id]=@dlg.get_element_value(id)} if @dlg and @dlg.visible?
  end#getIDvalues
  
  def setIDvalues()
    return nil if not @dlg or not @dlg.visible?
    ###
    @octane_render_binary=@values["octane_render_binary"]
	@project_path=@values["project_path"]
	@project_name=@values["project_name"]
	@native_unit_size=@values["native_unit_size"]
    
	@resolution=@values["resolution"]
    @width=@values["width"]
    @height=@values["height"]
    @percentage=@values["percentage"]
    
    @export_camera=@values["export_camera"]
    @active_camera=@values["active_camera"]
    @lens_aperture_radius=@values["lens_aperture_radius"]
    @lens_aperture_radius_value=@values["lens_aperture_radius_value"]
    @focal_depth=@values["focal_depth"]
    @focal_depth_value=@values["focal_depth_value"]
    @motion_blur=@values["motion_blur"]
    @motion_blur_value=@values["motion_blur_value"]
    
    @export_sun_direction=@values["export_sun_direction"]
    @light_source=@values["light_source"]
    
    @gpu_devices=@values["gpu_devices"]
    @gpu_device0=@values["gpu_device0"]
    @gpu_device1=@values["gpu_device1"]
    @gpu_device2=@values["gpu_device2"]
    @gpu_device3=@values["gpu_device3"]

    @image_output=@values["image_output"]
    @start_frame=@values["start_frame"]
    @end_frame=@values["end_frame"]
    @step=@values["step"]
    @pixel_samples=@values["pixel_samples"]
    @pixs=@values["pixs"]
    ###
  end#setIDvalues
  
  def saveIDvalues()
    return nil if not @dlg or not @dlg.visible?
    #puts 'saveIDvalues TO DO'
    ### save all settings to SUp or SKP
    Sketchup.write_default("Octane","octane_render_binary", @octane_render_binary.gsub(/\\/){'\\\\'})
    #
    @model.set_attribute("Octane", "project_path", @project_path)
    @model.set_attribute("Octane", "project_name", @project_name)
	#@native_unit_size dynamic
    
    @model.set_attribute("Octane", "resolution", @resolution)
    @model.set_attribute("Octane", "width", @width)
    @model.set_attribute("Octane", "height", @height)
    @model.set_attribute("Octane", "percentage", @percentage)
    
    @model.set_attribute("Octane", "export_camera", @export_camera)
    @model.set_attribute("Octane", "active_camera", @active_camera)
    @model.set_attribute("Octane", "lens_aperture_radius", @lens_aperture_radius)
    @model.set_attribute("Octane", "lens_aperture_radius_value", @lens_aperture_radius_value)
    @model.set_attribute("Octane", "focal_depth", @focal_depth)
    @model.set_attribute("Octane", "focal_depth_value", @focal_depth_value)
    @model.set_attribute("Octane", "motion_blur", @motion_blur)
    @model.set_attribute("Octane", "motion_blur_value", @motion_blur_value)
    
    @model.set_attribute("Octane", "export_sun_direction", @export_sun_direction)
    @model.set_attribute("Octane", "light_source", @light_source)
    
    @model.set_attribute("Octane", "gpu_devices", @gpu_devices)
    @model.set_attribute("Octane", "gpu_device0", @gpu_device0)
    @model.set_attribute("Octane", "gpu_device1", @gpu_device1)
    @model.set_attribute("Octane", "gpu_device2", @gpu_device2)
    @model.set_attribute("Octane", "gpu_device3", @gpu_device3)
    
    @model.set_attribute("Octane", "image_output", @image_output)

    @model.set_attribute("Octane", "start_frame", @start_frame)
    @model.set_attribute("Octane", "end_frame", @end_frame)
    @model.set_attribute("Octane", "step", @step)
    @model.set_attribute("Octane", "pixel_samples", @pixel_samples)
    @model.set_attribute("Octane", "pixs", @pixs)
    ###
  end#saveIDvalues
  
############## obj exporter part #############################
  
    def run_exporter(anim=false)
        ### set up mat list
        @all_mats=[nil] ### for 'default'
        mats=@model.materials.to_a
        @mats=[]
        @saved_names=[]
        mats.each{|mat|
          mat_name=mat.display_name.gsub(/ /, '_').gsub(/[^_0-9A-Za-z]/, '')
          saved_name=File.basename(mat_name)
          saved_name_uniq=self.make_name_unique(@saved_names, saved_name)
		  @saved_names << saved_name_uniq
          @mats << [mat, saved_name_uniq]
          @all_mats << mat
        }
        defns=@model.definitions
        img_mats=[]; @imgs=[]
        defns.each{|defn|
          next if not defn.image?
          face=nil
          defn.entities.each{|e|
            if e.class==Sketchup::Face
              face=e
              break
            end#if
          }
          next if not face
		  mat=face.material
          mat_name=mat.display_name.gsub(/ /, '_').gsub(/[^_0-9A-Za-z]/, '')
          saved_name=File.basename(mat_name)
          saved_name_uniq=self.make_name_unique(@saved_names, saved_name)
		  @saved_names << saved_name_uniq
          @imgs << [mat, saved_name_uniq]
          @all_mats << mat
        }
        self.settings(anim)
        self.do_export()
	end
	
	def settings(anim=false)
      if anim==false ### 1 frame render
        @obj_name=@project_name.tr("\\", "/")
        @obj_name=@obj_name+".obj"
        @obj_filepath=fileJoin(@project_path.tr("\\", "/"),@obj_name)
        @obj_filepath.tr!("/", "\\") if @pc
        @mtllib=@project_name.tr("\\", "/")+".mtl"
        @mtllib.tr!("/", "\\") if @pc
        @mtl_file=fileJoin(@project_path.tr("\\", "/"),@mtllib)
        @mtl_file.tr!("/", "\\") if @pc
		@textures_name=@project_name+"_"+(db("Textures"))
        @textures_path=fileJoin(@project_path, @textures_name)
        @textures_path.tr!("/", "\\") if @pc
		@v_counter=0
		@used_materials=[]
        @distorted=[]
        @dmats=[]
        #@smedges=[]
        #@soedges=[]
      else
        anim="" if anim=="-"
        @obj_name=@project_name.tr("\\", "/")
        @obj_name=@obj_name+"_anim"+anim
        @obj_name=@obj_name+".obj"
        @obj_filepath=fileJoin(@project_path.tr("\\", "/"),@obj_name)
        @obj_filepath.tr!("/", "\\") if @pc
        @mtllib=@project_name.tr("\\", "/")
        @mtllib.tr!("/", "\\") if @pc
        @mtllib=@mtllib+"_anim"+anim
        @mtllib=@mtllib+".mtl"
        @mtl_file=fileJoin(@project_path.tr("\\", "/"),@mtllib)
        @mtl_file.tr!("/", "\\") if @pc
		@textures_name=@project_name+"_"+(db("Textures"))
        @textures_path=fileJoin(@project_path, @textures_name)
        @textures_path.tr!("/", "\\") if @pc
		@v_counter=0
		@used_materials=[]
        @distorted=[]
        @dmats=[]
        #@smedges=[]
        #@soedges=[]
        @anims2go<< @obj_filepath
        @anims2go<< @mtl_file
      end#if
	end
    
	def do_export()
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Exporting OBJ & MTL Files...")))
		starttime=Time.now.to_f
	    @title=Sketchup.active_model.title
	    @title=(db("Untitled")) if not @title or @title==""
	    dirMkdir(@project_path) if not fileExist?(@project_path)
        ###
        self.export_start()
        ###
		endtime=(((Time.now.to_f - starttime)*10000).to_i/10000.0).to_f.to_s
		Sketchup.set_status_text((db("Octane Render:"))+" "+(db("OBJ & MTL Files Completed in"))+" #{endtime} "+(db("seconds")))
	end
    
    def export_start()
		@obj_file=fileNew(@obj_filepath, "w")
        @obj_file.puts "# Alias Wavefront OBJ "+(db("File Exported from SketchUp by 'Octane'"))
        @obj_file.puts "# (c) 2011 TIG & Refractive Software"
        @obj_file.puts "# "+"Units = meters"
        @obj_file.puts ""
		@obj_file.puts "mtllib #{@mtllib}"
        @obj_file.puts ""
		Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Making OBJ File...")))
		self.export_obj()
		@obj_file.close if @obj_file
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Exporting Textures...")))
		self.export_textures()
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Making MTL File...")))
		self.export_mtl_material()
        #@smedges.each{|e|e.smooth=true}
        #@soedges.each{|e|e.soft=true}
	end

	def export_obj()
        @img_mats=[]
		title=@title
		ents=Sketchup.active_model.entities.to_a
		objname=title+"_"+(db("Main"))
		Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Making Meshes...")))
        faces=ents.find_all{|e|e.class==Sketchup::Face}
	    if faces[0]
		    self.export_faces(faces, objname)
		end#if
		Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Meshes Completed.")))
		tr=Geom::Transformation.axes( (Geom::Point3d.new 0,0,0),(Geom::Vector3d.new(1,0,0)),(Geom::Vector3d.new(0,1,0)),(Geom::Vector3d.new(0,0,1)) )
        ###
	    ents.find_all{|e|e.class==Sketchup::Group}.each{|gp| self.export_group(gp, title, tr) }
	    ents.find_all{|e|e.class==Sketchup::ComponentInstance}.each{|ci| self.export_component_instance(ci, title, tr) }
        ents.find_all{|e|e.class==Sketchup::Image}.each{|ci| self.export_image(ci, title, tr) }
        ###
	end

	def export_group(gp, title="", tr=Geom::Transformation.new, mat=nil)
		return if gp.hidden? == true or gp.layer.visible? == false
		ot=tr * gp.transformation
		defmat=mat
		defmat=gp.material if gp.material
		@used_materials << defmat
		id=gp.entityID
		name="#{id}"
        name=name + "-" + gp.name if gp.name or gp.name=""
		name.gsub!(/[^\-_0-9A-Za-z]/, "_")
		objname=title + "_g_" + name
		if gp.entities.find{|e|e.class==Sketchup::Face}
			self.export_faces(gp.entities.find_all{|e|e.class==Sketchup::Face}, objname, ot, defmat)
		end
        ###
        gp.entities.find_all{|e|e.class==Sketchup::Image}.each{|f| self.export_image(f, title, ot) }
		gp.entities.find_all{|e| e.class==Sketchup::Group}.each{|f| self.export_group(f, title, ot, defmat) }
	    gp.entities.find_all{|e| e.class==Sketchup::ComponentInstance}.each{|f| self.export_component_instance(f, title, ot, defmat) }
        ###
	end
     
	def export_component_instance(ci, title="", tr=Geom::Transformation.new, mat=nil)
		return if ci.hidden? or not ci.layer.visible?
		ot=tr * ci.transformation
		defmat=mat
		defmat=ci.material if ci.material
		@used_materials << defmat
		id=ci.entityID
		name=ci.definition.name
		name="#{id}" if name.length == 0
		name.gsub!(/[^\-_0-9A-Za-z]/, "_")
		objname=title + "_c_" + name
		if ci.definition.entities.find{|e| e.class==Sketchup::Face }
			self.export_faces(ci.definition.entities.find_all{|e|e.class==Sketchup::Face}, objname, ot, defmat)
		end
		(ci.definition.entities.find_all{|e| e.class==Sketchup::Image}).each{|f| self.export_image(f, title, ot) }
        (ci.definition.entities.find_all{|e| e.class==Sketchup::Group}).each{|f| self.export_group(f, title, ot, defmat) }
	    (ci.definition.entities.find_all{|e| e.class==Sketchup::ComponentInstance}).each{|f| self.export_component_instance(f, title, ot, defmat) }
	end
    
    def export_image(img, title="", tr=Geom::Transformation.new)
		return if img.hidden? == true or img.layer.visible? == false
        defn=img.definition
        return if not defn
        face=nil
        defn.entities.each{|e|
          if e.class==Sketchup::Face
            face=e
            break
          end#if
        }
        return if not face
		ot=tr * img.transformation
		defmat=face.material
		@used_materials << defmat
		id=img.entityID
		name="#{id}"
		name.gsub!(/[^\-_0-9A-Za-z]/, "_")
		objname=title + "_i_" + name
        self.export_faces([face], objname, ot, defmat)
        ###
	end

	def export_faces(all_faces=[], objname="", tr=nil, defmat=nil)
        @all_mats.each{|mat|
			faces=all_faces.find_all{|face|face.material==mat}
			next if not faces[0]
            all_faces=all_faces-faces
            @used_materials << mat
            vs=[]
			nos=[]
			uvs=[]
			meshes=[]
			faces.each{|face|
				next if face.hidden? or not face.layer.visible?
                next if self.distorted?(face, objname, tr)
                #### we temporarily unsmooth/unsoft face edges to mesh it
                #face.edges.each{|e|
                  #if e.smooth?
                    #@smedges << e
                    #e.smooth=false
                  #end
                  #if e.soft?
                    #@soedges << e
                    #e.soft=false
                  #end
                #}
				mesh=face.mesh(5)###7=backs too
                ### reset smooth/soft at the end
				next if not mesh
                f_uvs=(1..mesh.count_points).map{|i|mesh.uv_at(i,1)}####1=front
                f_vs=[]
                f_vs=(1..mesh.count_points).map{|i|mesh.points[i-1]}
				f_nos=[]
                f_nos=(1..mesh.count_points).map{|i|mesh.normal_at(i)}
                f_vcount=1; f_vcount=vs.length + 1 if vs[0]
				meshes.concat(mesh.polygons.map{|p| [p.map{|px|(f_vs.index(mesh.points[(px.abs-1)]) + f_vcount)}] })
                vs.concat(f_vs) if f_vs
				uvs.concat(f_uvs) if f_uvs
				nos.concat(f_nos) if f_nos
			}#faces.each
			@meshes=meshes
			@nos=nos
            @vs=vs
			@uvs=uvs
            defn=faces[0].parent
            if not mat
              mat=defmat
              if defn != @model and defmat and defmat.texture
                ### re-map - it's on an Instance/Group
                ### we make exploded copy of it and map textures...
                tgp=@model.active_entities.add_group()
                tents=tgp.entities
                inst=tents.add_instance(defn, Geom::Transformation.new())
                inst.explode
                tents.to_a.each{|e|e.erase! if e.class==Sketchup::Face and e.material}
                tents.to_a.each{|e|e.material=mat if e.class==Sketchup::Face}
                self.export_faces(tents.find_all{|e|e.class==Sketchup::Face}, objname, tr, nil)
                tgp.erase!
              else
                self.export_obj_file(objname, tr, mat)
              end#if
            else
			  self.export_obj_file(objname, tr, mat)
            end#if
		}#@mats.each
        ###
        self.process_distorted() if @distorted
        ###
	end
    
    def distorted?(face=nil, objname="", tr=nil)
        return false if not face
        mat=face.material
        return false if not mat
        texture=mat.texture
        return false if not texture
        ### check for distortion
        mesh=face.mesh(5)###7=backs too
        uvs=[]
        f_uvs=(1..mesh.count_points).map{|i|mesh.uv_at(i,1)}####1=front
        uvs.concat(f_uvs) if f_uvs
        disto=false
        uvs.each{|uvq| disto=true if ((uvq.z.to_f)*100000).round != 100000 }
        @distorted << [face, objname, tr, mat] if disto
        return disto
    end#distorted?
    
    def process_distorted()
        @msg<<'.'
        Sketchup.set_status_text(@msg)
        @distorted.each{|a|
          face=a[0]
          objname=a[1]
          tr=a[2]
          mat=a[3]
          next if not face or not mat
          mesh=face.mesh(5)###7=backs too
		  next if not mesh
          vs=[]
		  nos=[]
		  uvs=[]
		  meshes=[]
          f_uvs=(1..mesh.count_points).map{|i|mesh.uv_at(i,1)}####1=front
          f_vs=[]
          f_vs=(1..mesh.count_points).map{|i|mesh.points[i-1]}
          f_nos=[]
          f_nos=(1..mesh.count_points).map{|i|mesh.normal_at(i)}
          f_vcount=1; f_vcount=vs.length + 1 if vs[0]
          meshes.concat(mesh.polygons.map{|p| [p.map{|px|(f_vs.index(mesh.points[(px.abs-1)]) + f_vcount)}] })
          ###
          vs.concat(f_vs) if f_vs
          #vs.each_with_index{|v,i|@model.entities.add_text(i.to_s, v)}
          uvs.concat(f_uvs) if f_uvs
         if uvs
          i=0; uvs.each{
            next if not uvs[i]
            j=i
            while j > 3
              j=j-4
            end
            uvs[i].z=1
            if j==0
              uvs[i].x= 1
              uvs[i].y= 1
            elsif j==1
              uvs[i].x= 0
              uvs[i].y= 0
            elsif j==2
              uvs[i].x= 1
              uvs[i].y= 0
            else
              uvs[i].x= 0
              uvs[i].y= 1
            end#if
            i+=1
          } if (vs[0].z >= vs[1].z and vs[0].z >= vs[2].z) or (vs[0].z <= vs[1].z and vs[0].z <= vs[2].z)
          i=0; uvs.each{
            next if not uvs[i]
            j=i
            while j > 3
              j=j-4
            end
            uvs[i].z=1
            if j==0
              uvs[i].x= 0
              uvs[i].y= 1
            elsif j==1
              uvs[i].x= 1
              uvs[i].y= 0
            elsif j==2
              uvs[i].x= 1
              uvs[i].y= 1
            else
              uvs[i].x= 0
              uvs[i].y= 0
            end#if
            i+=1
          } if (vs[0].z >= vs[1].z and vs[0].z <= vs[2].z)
          i=0; uvs.each{
            next if not uvs[i]
            j=i
            while j > 3
              j=j-4
            end
            uvs[i].z=1
            if j==0
              uvs[i].x= 1
              uvs[i].y= 0
            elsif j==1
              uvs[i].x= 0
              uvs[i].y= 1
            elsif j==2
              uvs[i].x= 0
              uvs[i].y= 0
            else
              uvs[i].x= 1
              uvs[i].y= 1
            end#if
            i+=1
          } if (vs[0].z <= vs[1].z and vs[0].z >= vs[2].z)
         end#if
          nos.concat(f_nos) if f_nos
          @vs=vs
		  @nos=nos
          @uvs=uvs
          @meshes=meshes
          dmat_name = self.export_distorted_obj(objname, tr, mat)
          @dmats << [face, mat, dmat_name]
        }#end distorted.each
        @distorted=[]
        ###
    end#process_distorted

	def export_obj_file(objname="", tr=nil, mat=nil)
		Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Processing Object File...")))
		ot=Geom::Transformation.new
		ot=tr if tr 
        if mat
		  mat_name="";(@mats+@imgs).each{|ar|(mat_name=ar[1];break) if ar[0]==mat}
        else
          mat_name=db("Default_Material")
		end
		return if not @meshes
		objname=objname.gsub(/ /, '_').gsub(/[^_0-9A-Za-z]/, '')
		if @meshes.length != 0 and @vs.length != 0
			if mat_name
				@obj_file.puts "g #{objname}-#{mat_name}"
				@obj_file.puts "usemtl #{mat_name}"
			else
				@obj_file.puts "g #{objname}-"+(db("Default_Material"))
				@obj_file.puts "usemtl "+(db("Default_Material"))
				mat_name=(db("Default_Material"))
			end#if
		    @v_counter=0 if not @v_counter
			@vs.each{|v|
				v=ot * v
				@obj_file.puts "v #{"%.6f" % v.x.to_m.to_f} #{"%.6f" % v.z.to_m.to_f} #{"%.6f" % (-v.y.to_m.to_f)}"
			}
			@nos.each{|vnor|
				nor=ot * vnor
				nor.normalize!
				@obj_file.puts "vn #{"%.6f" % (nor.x)} #{"%.6f" % (nor.z)} #{"%.6f" % (-nor.y)}"
			}
			@uvs.each{|uv|
				@obj_file.puts "vt #{"%.6f" % (uv.x)} #{"%.6f" % (uv.y)}"
			}
			@meshes.each{|mesh|
				f_str="f"
				mesh.each do |pg|
					pg.each{|j|
						k=j+@v_counter
						f_str += " #{k}/#{k}/#{k}"
					}
					@obj_file.puts f_str
				end
			}
			@obj_file.puts ""
			@v_counter += @vs.length if @vs[0]
		end
		@meshes=nil
		@uvs=nil
		@nos=nil
		@vts=nil
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Object Completed.")))
	end
    
    def export_distorted_obj(objname="", tr=nil, mat=nil)
        @msg<<'.'
        Sketchup.set_status_text(@msg)
		ot=Geom::Transformation.new
		ot=tr if tr 
        if mat
			mat_name=mat.display_name.gsub(/ /, '_').gsub(/[^_0-9A-Za-z]/, '')
            saved_name=File.basename(mat_name)
			saved_name_uniq=self.make_name_unique(@saved_names, saved_name)
			@saved_names << saved_name_uniq
            mat_name=saved_name_uniq # we always uniquify distorts
        else
            mat_name=db("Default_Material") ### should never happen !
		end
		return if not @meshes
		objname=objname.gsub(/ /, '_').gsub(/[^_0-9A-Za-z]/, '')
		if @vs.length != 0
            ###
			@obj_file.puts "g #{objname}-#{mat_name}"
			@obj_file.puts "usemtl #{mat_name}"
			###
		    @v_counter=0 if not @v_counter
			@vs.each{|v|
				v=ot * v
				@obj_file.puts "v #{"%.6f" % v.x.to_m.to_f} #{"%.6f" % v.z.to_m.to_f} #{"%.6f" % (-v.y.to_m.to_f)}"
			}
			@nos.each{|vnor|
				nor=ot * vnor
				nor.normalize!
				@obj_file.puts "vn #{"%.6f" % (nor.x)} #{"%.6f" % (nor.z)} #{"%.6f" % (-nor.y)}"
			}
			@uvs.each{|uv|
				@obj_file.puts "vt #{"%.6f" % (uv.x)} #{"%.6f" % (uv.y)}"
			}
			@meshes.each{|mesh|
				f_str="f"
				mesh.each do |pg|
					pg.each{|j|
						k=j+@v_counter
						f_str += " #{k}/#{k}/#{k}"
					}
					@obj_file.puts f_str
				end
			}
			@obj_file.puts ""
			@v_counter += @vs.length if @vs[0]
		end
		@meshes=nil
		@uvs=nil
		@nos=nil
		@vs=nil
        return mat_name
	end
    
	def make_texture_folder()
        begin
          dirMkdir(@textures_path) if not fileExist?(@textures_path)
        rescue
          UI.messagebox(@textures_path+" ??")
        end
        UI.messagebox(@textures_path+" ???")if not fileExist?(@textures_path)
	end
    
	def export_textures()
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Exporting Textures...")))
		self.make_texture_folder()
		temp_group=Sketchup.active_model.active_entities.add_group()
        tw=Sketchup.create_texture_writer
        @all_mats.each{|mat|
			next if not mat
			if mat.texture
                ### don't export unused & invisible materials for speed
                next if not @used_materials.include?(mat)
                @msg<<'.'
                Sketchup.set_status_text(@msg)
				temp_group.material=mat
                mat_texture_file=mat.texture.filename.tr("\\", "/")
                mat_texture_extn=File.extname(mat_texture_file)
                mat_name="";(@mats+@imgs).each{|ar|(mat_name=ar[1];break) if ar[0]==mat}
                mat_texture_name=mat_name+mat_texture_extn
                tpath=File.join(@textures_path, mat_texture_name)
                tw.load(temp_group)
                tw.write(temp_group, tpath)
                ###
			end#if
		}
		temp_group.erase! if temp_group.valid?
		Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Textures Exported.")))
	end

    
	def export_mtl_material()
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("Making MTL File...")))
        ffcol=@model.rendering_options["FaceFrontColor"]
		mtl_file=fileNew(@mtl_file,"w")
        mtl_file.puts "# Alias Wavefront MTL "+(db("File Exported from SketchUp by 'Octane'"))
        mtl_file.puts "# (c) 2011 TIG & Refractive Software"
        mtl_file.puts "# "+"Made for '"+@obj_name+"'"
        mtl_file.puts ""
		mtl_file.puts "newmtl "+(db("Default_Material"))
		mtl_file.puts "Ka 0.000000 0.000000 0.000000"
		mtl_file.puts "Kd " + ffcol.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
		mtl_file.puts "Ks 0.000000 0.000000 0.000000"
        mtl_file.puts "d 1.000000"
		mtl_file.puts ""
        saved_names=[]
		@used_materials.uniq!
		@used_materials.each{|mat|
			next if not mat
            mat_name=mat.display_name.gsub(/ /, '_').gsub(/[^_0-9A-Za-z]/, '')
            saved_name=fileBasename(mat_name)
			saved_name_uniq=self.make_name_unique(saved_names, saved_name)
			saved_names << saved_name_uniq
            mat_name=saved_name_uniq
			if mat and mat.texture
                texture_extn=fileExtname(mat.texture.filename)
				texture_path=(@textures_name + "/" + mat_name + texture_extn)
			end
			mtl_file.puts "newmtl #{mat_name}"
			if not mat.use_alpha?
				mtl_file.puts "Ka 0.000000 0.000000 0.000000"
				mtl_file.puts "Kd " + mat.color.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
				mtl_file.puts "Ks 0.000000 0.000000 0.000000"
				mtl_file.puts "d 1.000000"
				mtl_file.puts "map_Kd #{texture_path}" if texture_path
			else ### it's transparent
				mtl_file.puts "Ka 0.000000 0.000000 0.000000"
				mtl_file.puts "Kd " + mat.color.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
				mtl_file.puts "Ks 0.000000 0.000000 0.000000"
				mtl_file.puts "d #{"%.6f" % mat.alpha }"### it's NOT (1 - mat.alpha) !!!
				mtl_file.puts "map_Kd #{texture_path}" if texture_path
			end#if
            mtl_file.puts ""
		}
		### do any distorts
        self.distorted_mtl(mtl_file)
        ###
		mtl_file.puts "#EOF"
        ###
		mtl_file.flush
		mtl_file.close
        ###
        Sketchup.set_status_text((db("Octane Render:"))+" "+(db("MTL File Completed.")))
	end
    
    def distorted_mtl(mtl_file)
        return nil if not @dmats[0]
        ###
        mtl_file.puts("#Distorted-Textures\n\n")
        ###
        @dmats.each{|ar|
            @msg<<'.'
            Sketchup.set_status_text(@msg)
            face=ar[0]
            mat=ar[1]
            mat_name=ar[2]
			if mat and mat.texture
                texture_extn=File.extname(mat.texture.filename)
				texture_path=File.join(@textures_name, mat_name + texture_extn)
			end
			mtl_file.puts "newmtl #{mat_name}"
			if not mat.use_alpha?
				mtl_file.puts "Ka 0.000000 0.000000 0.000000"
				mtl_file.puts "Kd " + mat.color.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
				mtl_file.puts "Ks 0.000000 0.000000 0.000000"
				mtl_file.puts "d 1.000000"
				mtl_file.puts "map_Kd #{texture_path}" if texture_path
			else ### it's transparent
				mtl_file.puts "Ka 0.000000 0.000000 0.000000"
				mtl_file.puts "Kd " + mat.color.to_a[0..2].collect{|c| "%.6f" % ((c.to_f/255)) }.join(" ")
				mtl_file.puts "Ks 0.000000 0.000000 0.000000"
				mtl_file.puts "d #{"%.6f" % mat.alpha }"### it's NOT (1 - mat.alpha) !!!
				mtl_file.puts "map_Kd #{texture_path}" if texture_path
			end#if
            mtl_file.puts ""
            ###
            self.export_distorted_texture(face, texture_path)
            ###
        }
    end#distorted_mtl
    
    def export_distorted_texture(face, texture_path)
        tpath=File.join(@project_path, texture_path)
        tw=Sketchup.create_texture_writer
        tw.load(face, true)
        tw.write(face, true, tpath)
    end#export_distorted_texture
    
	def make_name_unique(saved_names=[], saved_name="")
		if saved_names.include?(saved_name)
			counter=1
			while counter < 10000
				new_name=fileBasename(saved_name, ".*") + counter.to_s + fileExtname(saved_name)
				return new_name if not saved_names.include?(new_name)
				counter += 1
			end
		end
		return saved_name
	end
############## end of exporter code #########################


### start folder_browser code ###
    def startfb(dir=nil, input=nil)
        @octane_render_folder_browser_result = nil
        begin
          dirChdir(dir) if dir
        rescue
          UI.messagebox((db("Can't Change Directory to"))+":\n"+dir)
        end
		@dlgfb=UI::WebDialog.new((db("Browse for Folder >> Highlight >> OK")),false,"Octane_FolderBrowser",350,350,50,50,false)
		file=fileJoin(fileDirname(__FILE__), "OctaneFolderBrowser/folder_browser.html")
		@dlgfb.set_file(file)
        @dlgfb.min_width=350
        @dlgfb.add_action_callback("jserr"){|d,a|UI.messagebox((db("ERROR"))+":\n"+a) }
        @dlgfb.add_action_callback("mkdir"){|d,a|self.newfolder(dirPwd) }
        @dlgfb.add_action_callback("cancel"){|d,a|d.close }
        @dlgfb.add_action_callback("alert"){|d,a|UI.messagebox(db("Highlight a Folder!")) }
		@dlgfb.add_action_callback("folder"){|d,a|
            ### 'a' is a csv string of char codes - convert to ansi string
            chars=a.split(",")
            chars.compact!
            chars.delete("")
            chars.length.times{|i|chars[i]=chars[i].to_i}
            txt=chars.map{|c|c.chr}.join
            begin
			  dirChdir(txt)
              pwd=dirPwd
              d.execute_script('clear_list()')
			  self.populatefb(pwd)
            rescue
              UI.messagebox((db("Can't Change Directory to"))+":\n"+txt)
            end
		}
        ###
		@dlgfb.add_action_callback("use_this"){|d,a|
            ### 'a' is a csv string of char codes, so Unicode comes through.
            chars=a.split(",")
            chars.compact!
            chars.delete("")
            chars.length.times{|i|chars[i]=chars[i].to_i}
            txt=chars.map{|c|c.chr}.join
            @octane_render_folder_browser_result = txt
            special=false
            special=true if @pc and @octane_render_folder_browser_result.length != @octane_render_folder_browser_result.unpack("C*").pack("U*").length
            if special
              UI.messagebox((db("Octane Render:"))+"\n"+(db("Cannot continue without a valid 'Project Path'!"))+"\n"+(db("NO accented or special-characters allowed.")))
            else
              d.close
              if input == "image"
                self.update_image()
              elsif input == "project"
                self.update_project()
              end#if
            end#if
		}
        ###
        @dlgfb.set_on_close{@dlgfb=nil}### nullify itself!
        @dlgfb.show_modal{self.populatefb()} #sleep(0.1); 
	end
    ###
    def newfolder(dir=nil)
        results=UI.inputbox([((db("New Folder Name"))+" ")],[""],"")
        return nil if not results
        newDir=results[0]
        return nil if not newDir or newDir==""
        newDir.gsub!(/[^_A-Za-z0-9]/, '')### strip out
        if not newDir or newDir==""
          UI.messagebox(db("Folder's Name Must Contain Only _A-Za-z0-9"))
          return nil
        end#if
        dir=dirPwd if not dir
        begin
          dirChdir(dir)
        rescue
          UI.messagebox((db("Can't Change Directory to"))+":\n"+dir)
          return nil
        end
        if fileExist?(newDir)
          UI.messagebox(db("It Already Exists!"))
        else
          dirMkdir(newDir)
          @dlgfb.execute_script('clear_list()')
          self.populatefb(dir, newDir)
        end#if
    end
    ###
	def populatefb(dir=nil, newDir=nil)
        ### translate buttons
        js=("document.getElementById(\"new_button\").setAttribute(\"value\",\""+(db("New Folder"))+"\");")
        @dlgfb.execute_script(js)
        js=("document.getElementById(\"cancel_button\").setAttribute(\"value\",\""+(db("Cancel"))+"\");")
        @dlgfb.execute_script(js)
        js=("document.getElementById(\"ok_button\").setAttribute(\"value\",\""+(db("OK"))+"\");")
        @dlgfb.execute_script(js)
        ###
		list=[]
		dir=dirPwd if not dir
        begin
		  dirChdir(dir)
        rescue
          UI.messagebox((db("Can't Change Directory to"))+":\n"+dir)
        end
		pwd=dirPwd
		@dlgfb.execute_script('set_location("'+pwd+'");')
		Dir[pwd+"/*"].sort.each{|file|
			next unless fileDirectory?(file)
			list << file if fileBasename(file) != newDir
		}
		scr='appendDiv("../");'
		@dlgfb.execute_script(scr)
        scr='appendDiv("'+newDir+'");' if newDir
		@dlgfb.execute_script(scr) if newDir
		list.each{|name|
			name=fileBasename(name)
			scr='appendDiv("'+name+'");'
			@dlgfb.execute_script(scr)
		}
	end

####################### end fb ###
  
end#class Octane ##########################################################
###

### make shortcut to tool #################################################
def octane()
  Sketchup.active_model.select_tool(Octane.new)
end#def octane
###


### add menu item etc #####################################################
if not file_loaded?(File.basename(__FILE__))
  ### deBabelized
  text0 = Octane::db("Octane Render")
  text1 = Octane::db("Octane Render...")
  text2 = Octane::db("Opens the 'Octane Render' dialog to export/render the model...")
  ### make command
  cmd=UI::Command.new(text1){octane}
  cmd.tooltip=(text1)
  cmd.status_bar_text=(text2)
  ### menu
  UI.menu("File").add_item(cmd)
  ### context menu ### disabled by default
  #UI.add_context_menu_handler do |menu|
    #menu.add_item(cmd)
  #end#do menu
  ### toolbar
  cmd.small_icon=File.join(File.dirname(__FILE__), "OctaneToolbar16x16.png")
  cmd.large_icon=File.join(File.dirname(__FILE__), "OctaneToolbar24x24.png")
  toolbar=UI::Toolbar.new(text0)
  toolbar.add_item(cmd)
  toolbar.restore if toolbar.get_last_state==TB_VISIBLE
  ###
end#if
file_loaded(File.basename(__FILE__))
###